#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <gsf/gsf.h>
#include <gdk_imlib.h>

static GsfSpriteGfx *
load_mbf(char *fn, gboolean f)
{
    GsfSpriteGfx *gfx;

    GdkPixmap *p;
    GdkBitmap *m;
    
    GdkImlibImage *image;
    
    image = gdk_imlib_load_image(fn);
    if (f)
	gdk_imlib_flip_image_vertical(image);
    gdk_imlib_render(image, image->rgb_width, image->rgb_height);
    p = gdk_imlib_move_image(image);
    m = gdk_imlib_move_mask(image);
    
    if (!p)
	g_error("No image!");
    if (!m) {
	g_warning("No mask!");
	/* do something smart */
    }
    if (!m)
	g_error("Really no mask!!");
    
    gfx = gsf_spritegfx_new();
    gsf_spritegfx_set(gfx, p, m);
    gfx->hotx = gfx->width/2;
    gfx->hoty = gfx->height/2;

    return gfx;
}

static GsfSpriteGfx *
load(char *fn)
{
    return load_mbf(fn, FALSE);
}

static GsfSpriteGfx *
loadflip(char *fn)
{
    return load_mbf(fn, TRUE);
}

static GsfSpriteGfx **
loadseq(char *fn, int num)
{
    GsfSpriteGfx **im;
    GString *filename;
    gint i;

    filename = g_string_new(NULL);

    im = g_new(GsfSpriteGfx *, num);
    for (i = 0; i < num; i++) {
	g_string_sprintf(filename, fn, i);
	im[i] = load(filename->str);
    }
    g_string_free(filename, TRUE);
    
    return im;
}

static void
freeseq(GsfSpriteGfx **gfxseq, int num)
{
/*     gint i; */
/*     for (i = 0; i < num; i++) */
/* 	gtk_object_unref(GTK_OBJECT(gfxseq[i])); */
    g_free(gfxseq);
}

gint animator;
gint max;

gint
animate(gpointer data)
{
    static gint wait = 15;
    GsfSpriteView *view;
    GsfSpriteField *field;
    GSList *iter;
    g_return_val_if_fail(data != NULL, FALSE);

    if (wait > 0) {
	wait--;
	return TRUE;
    }
    
    view = GSF_SPRITEVIEW(data);
    if (GTK_OBJECT_DESTROYED(view))
	return FALSE;
    
    field = view->field;
    g_return_val_if_fail(field != NULL, FALSE);

    iter = field->all_objects;
    while (iter) {
	if (GSF_IS_MOVEPMSPRITE(iter->data))
	    gsf_movepmsprite_move(GSF_MOVEPMSPRITE(iter->data));
	iter = iter->next;
    }
    
    gsf_spritefield_update(field);

    {
	GSList *list;
	GSList *iter;
	gchar *arr;
	gint i;

	arr = g_new(gchar, (max+1)*max + 1);
	memset(arr, '.', (max+1)*max);
	for (i = 0; i < max; i++)
	    arr[i*(max+1) + max] = '\n';
	arr[max*(max+1)] = '\0';
	
	iter = list = gsf_spritefield_get_collisions(field);
	while (iter) {
	    GSList *tmp;
	    GsfBaseSprite *b;
	    gint r, c;

	    b = GSF_BASESPRITE(iter->data);
	    tmp = gsf_basesprite_at(b);
	    r = (gint)gtk_object_get_user_data(GTK_OBJECT(b)) - 1;
	    arr[r*(max+1) + r] = 'X';
	    while (tmp) {
		c = (gint)gtk_object_get_user_data(GTK_OBJECT(tmp->data)) - 1;
		arr[r*(max+1) + c] = 'X';
		tmp = tmp->next;
	    }
	    
	    iter = iter->next;
	}
	puts(arr);

	g_slist_free(list);
	g_free(arr);
    }

    return TRUE;
}

void
quit(void)
{
    gtk_timeout_remove(animator);
    gtk_main_quit();
}

#define WW 600
#define HH 400

GtkWidget *
get_view(void)
{
    GtkWidget *view;
    GsfSpriteField *field;
    GsfSpriteGfx **ball;
    GsfSpriteGfx *tmp;
    GsfPixmapSprite *ps;
    GsfPixmapSprite *ps2;
    GsfPixmapSprite *bl;
    GsfMovePmSprite *ms;
    gint i;

    i = 0;
    field = gsf_spritefield_new(WW, HH, 20);
    {
	GdkPixmap *p;
	gdk_imlib_load_file_to_pixmap("backgrnd.png", &p, NULL);
	gsf_spritefield_setbgpixmap(field, p);
/* 	gdk_pixmap_unref(p); */
    }
    
    view = gsf_spriteview_new();
    gsf_spriteview_setfield(GSF_SPRITEVIEW(view), field);
    
    tmp = load("test.png");
    ps2 = ps = gsf_pixmapsprite_new();
    gtk_object_set_user_data(GTK_OBJECT(ps), (gpointer)++i);
    gsf_pixmapsprite_set(ps, 110, 210, 0, &tmp, 1, field, 0);
/*     gtk_object_unref(GTK_OBJECT(ps)); */
/*     gtk_object_unref(GTK_OBJECT(tmp)); */

    tmp = load("blank.png");
    bl = ps = gsf_pixmapsprite_new();
    gtk_object_set_user_data(GTK_OBJECT(ps), (gpointer)++i);
    gsf_pixmapsprite_set(ps, 300, 200, 0, &tmp, 1, field, 0);
/*     gtk_object_unref(GTK_OBJECT(tmp)); */

    
/*     ball = loadseq("ping%d.png", 64); */
/*     obj = gsf_movepmsprite_new(); */
/*     ms = GSF_MOVEPMSPRITE(obj); */
/*     ps = GSF_PIXMAPSPRITE(obj); */
/*     gtk_object_set_user_data(obj, (gpointer)++i); */
/*     gsf_pixmapsprite_set(ps, 220, 240, 0, ball, 64, field, -11); */
/*     gsf_movepmsprite_setmove(ms, 0, 0, 1); */
/*     gtk_object_unref(obj); */
/*     freeseq(ball, 64); */
    
    printf("XXXXXXX %p\n", ps);

    ball = loadseq("sprite%d.png", 32);
    for (; i<3;) {
	gint dx, dy, df, w, h;
	ms = gsf_movepmsprite_new();
	ps = GSF_PIXMAPSPRITE(ms);
	gtk_object_set_user_data(GTK_OBJECT(ms), (gpointer)++i);


	gsf_pixmapsprite_set(ps,
			     (rand()%(WW-100)) + 50,
			     (rand()%(HH-100)) + 50,
			     (rand()%32),
			     ball, 32,
			     field, (rand()%11) - 5);
	dx = (rand()%5)+1;
	dy = (rand()%5)+1;
	df = (rand()%5)+1;
	gsf_movepmsprite_setmove(ms,
			         rand()&1?dx:-dx,
			         rand()&1?dy:-dy,
			         rand()&1?df:-df);
	gsf_pixmapsprite_getsize(ps, &w, &h);
	gsf_movepmsprite_setbounds(ms,
				   w/2, h/2,
				   WW - w/2, HH - h/2,
				   GSF_MOVEPMSPRITE_BOUNCE);
/* 	gtk_object_unref(GTK_OBJECT(ms)); */
    }
    max = i;
    for (i = 0; i < 50; i++) {
	gsf_spritegfx_pastegfx(bl->gfx[bl->currentframe],
			       ball[rand()%32],
			       (rand()%(WW+100)) - (WW+100)/2,
			       (rand()%(HH+100)) - (HH+100)/2);
    }
/*     gtk_object_unref(GTK_OBJECT(bl)); */
    freeseq(ball, 32);

/*     gtk_object_unref(GTK_OBJECT(field)); */
    return view;
}

#define rdtsc() ({ long long v; asm volatile("rdtsc\n":"=A"(v):/**/); v; })

int
main(int argc, char *argv[])
{
    GtkWidget *window;
    GtkWidget *view;

    gtk_init(&argc, &argv);
    gdk_imlib_init();
    gtk_widget_push_visual(gdk_imlib_get_visual());
    gtk_widget_push_colormap(gdk_imlib_get_colormap());

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_signal_connect(GTK_OBJECT(window), "destroy",
		       GTK_SIGNAL_FUNC(quit), NULL);
    gtk_container_border_width (GTK_CONTAINER(window), 10);

    view = get_view();
    gtk_container_add(GTK_CONTAINER(window), view);

    gtk_widget_show(view);
    gtk_widget_show(window);
    
    animator = gtk_timeout_add(50, animate, view);
    
    gtk_main();
  
    gtk_type_describe_tree(gtk_object_get_type(), TRUE);
    g_mem_profile();

/*     { */
/* 	GsfSpriteGfx *tmp; */
/* 	gint i; */
/* 	{ */
/* 	    GdkPixmap *p; */
/* 	    GdkBitmap *m; */
	   
/* 	    gdk_imlib_load_file_to_pixmap("testcoll.png", &p, &m); */
/* 	    if (!p) g_error("No image!"); */
/* 	    if (!m) g_error("No mask!"); */
    
/* 	    tmp = GSF_SPRITEGFX(gsf_spritegfx_new()); */
/* 	    gsf_spritegfx_set(tmp, p, m); */
/* 	} */
/* 	printf ("X = %d\n", gsf_spritegfx_collides(tmp, 9, -9, tmp, 0, 0)); */
/* 	fflush(stdout); */
	/* Check collision (checkboard) */
/* 	{ */
/* 	    gint w = tmp->collmask.width; */
/* 	    gint h = tmp->collmask.height; */
/* 	    long long t = -rdtsc(); */
/* 	    for (i = 0; i < 10000; i++) { */
/* 		gint dx = (rand()%(4*w+1)) - 2*w; */
/* 		gint dy = (rand()%(4*h+1)) - 2*h; */

/* 		gboolean yup = dx>-w && dx<w && dy>-h && dy<h && !((dx+dy) & 1); */
/* 		gboolean col = gsf_spritegfx_collides(tmp, dx, dy, tmp, 0, 0); */

/* 		if (!((w+h) & 1) && ((dx == -w+1 && dy == h-1) || (dx == w-1 && dy == -h+1))) */
/* 		    yup = 0; */
/* 		if (yup != col) { */
/* 		    printf ("Uh oh! %d != %d for %d, %d\n", */
/* 			    yup, col, dx, dy); */
/* 		} */
/* 	    } */
/* 	    t += rdtsc(); */
/* 	    printf ("Time = %Ld\n", t); */
/* 	} */
/*     } */

    
    return 0;
}
