#include "loadgfx.h"

#include <stdlib.h>
#include <png.h>
#include <gdk_imlib.h>

#include <string.h>

GsfSpriteGfx *
create_blank_gfx(guint width, guint height)
{
    GsfSpriteGfx *gfx;
    GdkPixmap *pic;
    GdkBitmap *mask;
    GdkImlibImage *image;
    unsigned char *data;
    GdkImlibColor trans = {r:0, g:0, b:0};

    data = g_new0(unsigned char, width*height*3);
    image = gdk_imlib_create_image_from_data(data, NULL, width, height);
    gdk_imlib_set_image_shape(image, &trans);
    gdk_imlib_changed_image(image);
    if (!gdk_imlib_render(image, width, height))
	g_error("Could not create pixmap");
    pic = gdk_imlib_move_image(image);
    mask = gdk_imlib_move_mask(image);
    gdk_imlib_kill_image(image);
    
    gfx = gsf_spritegfx_new();
    gsf_spritegfx_set(gfx, pic, mask);
    return gfx;
}

GsfSpriteGfx *
create_filled_gfx(guint width, guint height)
{
    GsfSpriteGfx *gfx;
    GdkPixmap *pic;
    GdkBitmap *mask;
    GdkImlibImage *image;
    unsigned char *data;
    GdkImlibColor trans = {r:0, g:0, b:0};

    data = g_new(unsigned char, width*height*3);
    memset(data, 255, width*height*3);
    
    image = gdk_imlib_create_image_from_data(data, NULL, width, height);
    gdk_imlib_set_image_shape(image, &trans);
    gdk_imlib_changed_image(image);
    if (!gdk_imlib_render(image, width, height))
	g_error("Could not create pixmap");
    pic = gdk_imlib_move_image(image);
    mask = gdk_imlib_move_mask(image);
    gdk_imlib_kill_image(image);
    
    gfx = gsf_spritegfx_new();
    gsf_spritegfx_set(gfx, pic, mask);
    return gfx;
}

GsfSpriteGfx *
create_bordered_gfx(guint width, guint height,
		    guint left, guint top,
		    guint right, guint bottom)
{
    GsfSpriteGfx *gfx;

    gint x, y;
    GsfCollisionMask *mask;
    guint32 *data;

    width += left + right;
    height += top + bottom;
    
    gfx = create_blank_gfx(width, height);
    gfx->hotx = left;
    gfx->hoty = top;

    mask = gfx->collmask;
    data = mask->data;

    for (y = 0; y < top; y++) {
	guint32 *tmp = data + y*mask->wpl;
	for (x = 0; x < width; x++)
	    tmp[x >> 5] |= 0x80000000ul >> (x & 31);
    }
    for (y = height-bottom; y < height; y++) {
	guint32 *tmp = data + y*mask->wpl;
	for (x = 0; x < width; x++)
	    tmp[x >> 5] |= 0x80000000ul >> (x & 31);
    }
    for (x = 0; x < left; x++) {
	guint32 *tmp = data + top*mask->wpl + (x >> 5);
	guint32 *end = tmp + (height - bottom - top)*mask->wpl;
	guint32 orval = 0x80000000ul >> (x & 31);
	while (tmp < end) {
	    *tmp |= orval;
	    tmp += mask->wpl;
	}
    }
    for (x = width-right; x < width; x++) {
	guint32 *tmp = data + top*mask->wpl + (x >> 5);
	guint32 *end = tmp + (height - bottom - top)*mask->wpl;
	guint32 orval = 0x80000000ul >> (x & 31);
	while (tmp < end) {
	    *tmp |= orval;
	    tmp += mask->wpl;
	}
    }
    
    return gfx;
}

static gboolean
readbuf(gzFile in, voidp buf, unsigned len)
{
    int r;

    while (len > 0 && (r = gzread(in, buf, len)) > 0 ) {
	len -= r;
	buf += r;
    }

    return (len == 0);
}

static gboolean
initpngreader(png_structp *png_ptr, png_infop *info_ptr, png_infop *einfo_ptr)
{
    g_return_val_if_fail(png_ptr && info_ptr && einfo_ptr, FALSE);
    
    *png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
    if (!*png_ptr)
	return FALSE;
    
    *info_ptr = png_create_info_struct(*png_ptr);
    if (!*info_ptr) {
        png_destroy_read_struct(png_ptr, (png_infopp)NULL, (png_infopp)NULL);
        return FALSE;
    }

    *einfo_ptr = png_create_info_struct(*png_ptr);
    if (!*einfo_ptr)
    {
        png_destroy_read_struct(png_ptr, info_ptr, (png_infopp)NULL);
        return FALSE;
    }

    return TRUE;
}


static void
zread_png(png_structp png_ptr, png_bytep data, png_size_t length)
{
    gzFile in;

    in = (gzFile)png_get_io_ptr(png_ptr);

    if (!readbuf(in, data, length))
	png_error(png_ptr, "Read Error");
}

static gboolean
loadpngdata(gzFile in,
	    GdkImlibImage **pic,
	    GdkImlibImage **mask,
	    gint *hotx, gint *hoty)
{
    png_structp   png_ptr;
    png_infop     info_ptr;
    png_infop     einfo_ptr;
    png_uint_32   w, h;
    png_uint_32   rowbytes;
    png_bytep     row;
    int           bit_depth;
    int           color_type;
    int           interlace_type;
    double        screen_gamma;
    double        gamma;
    png_uint_32   offset_x, offset_y;
    int           unit_type;
    const char   *gamma_str;
    gint          counter;
    unsigned char *pdata, *mdata;
    GdkImlibColor trans = {r:0, g:0, b:0};
    
    g_return_val_if_fail(in != NULL, FALSE);

    if (!initpngreader(&png_ptr, &info_ptr, &einfo_ptr)) {
	g_warning("Could not initialize png reader structures");
	return FALSE;
    }
    
    if (setjmp(png_ptr->jmpbuf)) {
	g_warning("Error reading png file");
	png_destroy_read_struct(&png_ptr, &info_ptr, &einfo_ptr);
	return FALSE;
    }

    png_set_read_fn(png_ptr, (voidp)in, &zread_png);

    png_read_info(png_ptr, info_ptr);
    png_get_IHDR(png_ptr, info_ptr, &w, &h, &bit_depth, &color_type,
		 &interlace_type, NULL, NULL);

    if (color_type == PNG_COLOR_TYPE_PALETTE)
	png_set_expand(png_ptr);
    if (color_type == PNG_COLOR_TYPE_GRAY)
	png_set_expand(png_ptr);
    if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
	png_set_expand(png_ptr);
    if (bit_depth == 16)
	png_set_strip_16(png_ptr);
    if (color_type == PNG_COLOR_TYPE_GRAY
	|| color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
	png_set_gray_to_rgb(png_ptr);
    png_set_filler(png_ptr, 0xFF, PNG_FILLER_AFTER);

    if ((gamma_str = getenv("SCREEN_GAMMA")) != NULL) {
	screen_gamma = atof(gamma_str);
    } else
	screen_gamma = 2.0;

    if (png_get_gAMA(png_ptr, info_ptr, &gamma))
	png_set_gamma(png_ptr, screen_gamma, gamma);
    else
	png_set_gamma(png_ptr, screen_gamma, 0.50);

    png_read_update_info(png_ptr, info_ptr);

    /* Interlace support not to hard, but... */
    if (interlace_type != PNG_INTERLACE_NONE)
	g_error("Interlace not supported (yet)");

    pdata = pic ? g_new(unsigned char, w*h*3) : NULL;
    mdata = mask ? g_new(unsigned char, w*h*3) : NULL;
    rowbytes = png_get_rowbytes(png_ptr, info_ptr);
    row = g_new(png_byte, rowbytes);
    for (counter = 0; counter < h; counter++) {
	unsigned char *pdst, *mdst;
	png_byte *src, *end;
	
	png_read_row(png_ptr, row, NULL);

	src = row;
	end = src + w*4;
	if (pic && mask) {
	    pdst = pdata + w*counter*3;
	    mdst = mdata + w*counter*3;
	    while (src < end) {
		*pdst++ = *src++;
		*pdst++ = *src++;
		*pdst++ = *src++;

		mdst[0] = mdst[1] = mdst[2] = (*src++ < 0x80 ? 0 : 0xFF);
		mdst += 3;
	    }
	} else if (pic) {
	    pdst = pdata + w*counter*3;
	    while (src < end) {
		*pdst++ = *src++;
		*pdst++ = *src++;
		*pdst++ = *src++;
		src++;
	    }
	} else if (mask) {
	    mdst = mdata + w*counter*3;
	    while (src < end) {
		*src += 3;
		mdst[0] = mdst[1] = mdst[2] = (*src++ < 0x80 ? 0 : 0xFF);
		mdst += 3;
	    }
	}
    }
    g_free(row);

    if (pic) {
	*pic = gdk_imlib_create_image_from_data(pdata, NULL, w, h);
	g_free(pdata);
	gdk_imlib_changed_image(*pic);
    }
    if (mask) {
	*mask = gdk_imlib_create_image_from_data(mdata, NULL, w, h);
	g_free(mdata);
	gdk_imlib_set_image_shape(*mask, &trans);
	gdk_imlib_changed_image(*mask);
    }
    
    png_read_end(png_ptr, einfo_ptr);

    if ((!png_get_oFFs(png_ptr, info_ptr, &offset_x, &offset_y, &unit_type)
	&& !png_get_oFFs(png_ptr, einfo_ptr, &offset_x, &offset_y, &unit_type))
	|| unit_type != PNG_OFFSET_PIXEL) {
	offset_x = 0;
	offset_y = 0;
    }

    if (hotx) *hotx = offset_x;
    if (hoty) *hoty = offset_y;

    png_destroy_read_struct(&png_ptr, &info_ptr, &einfo_ptr);

    return TRUE;
}


void
loadpng_pixmap(gzFile in,
	       GdkPixmap **mypix, GdkBitmap **mymask,
	       gint *hotx, gint *hoty)
{
    GdkImlibImage *pic;
    GdkImlibImage *mask;
    
    if (!loadpngdata(in,
		     mypix ? &pic : NULL, mymask ? &mask : NULL,
		     hotx, hoty))
	g_error("Could not read png image");

    if (mypix) {
	if (!gdk_imlib_render(pic, pic->rgb_width, pic->rgb_height))
	    g_error("Could not create pixmap");

	*mypix = gdk_imlib_move_image(pic);
	gdk_imlib_kill_image(pic);
    }

    if (mymask) {
	if (!gdk_imlib_render(mask, mask->rgb_width, mask->rgb_height))
	    g_error("Could not create pixmap");
	*mymask = gdk_imlib_move_mask(mask);
	gdk_imlib_kill_image(mask);
    }
}

GsfSpriteGfx *
loadpng(gzFile in)
{
    GsfSpriteGfx *gfx;
    GdkPixmap *mypix;
    GdkBitmap *mymask;
    gint hotx, hoty;

    loadpng_pixmap(in, &mypix, &mymask, &hotx, &hoty);

    gfx = gsf_spritegfx_new();
    gsf_spritegfx_set(gfx, mypix, mymask);
    gfx->hotx = hotx;
    gfx->hoty = hoty;
    
    return gfx;
}

GsfSpriteGfx **
loadpngtar(gzFile in, gint *n)
{
    GsfSpriteGfx **g;
    gint p;

    g_return_val_if_fail(in != NULL && n != NULL, NULL);

    p = G_MAXINT;
    g = loadpngntar(in, &p);
    if (g)
	*n = p;

    return g;
}


static gboolean
iszero(guchar *buf, guint len)
{
    guchar *ebuf = buf + (len & ~3);

    while (buf < ebuf)
	if (*((gulong *)buf)++)
	    return FALSE;
    if (len & 3) {
	ebuf += len & 3;
	while (buf < ebuf)
	    if (*buf++)
		return FALSE;
    }
    return TRUE;
}

static gulong
getoctnum(guchar *buf, int num)
{
    gulong val = 0;

    while (*buf && *buf == ' ' && num)
	buf++, num--;
    while (*buf >= '0' && *buf <= '7' && num--)
	val = (val << 3) + (*buf++ - '0');

    return val;
}

static gboolean
istarheader(guchar tarheader[512])
{
    gulong chksum;
    guchar *p;
    guchar *e;

    if (strncmp("ustar", &tarheader[257], 5) != 0)
	return FALSE;

    chksum = 0;
    p = &tarheader[0];
    e = &tarheader[148];
    while (p < e)
	chksum += *p++;
    p += 8;
    chksum += 8*' ';
    e = &tarheader[512];
    while (p < e)
	chksum += *p++;
    if (chksum != getoctnum(&tarheader[148], 8))
	return FALSE;

    return TRUE;
}

GsfSpriteGfx **
loadpngntar(gzFile in, gint *n)
{
    gint max;
    gint capacity;
    gint num;
    gint mod;
    GsfSpriteGfx **res;
    guchar tarheader[512];

    g_return_val_if_fail(in != NULL && n != NULL && *n > 0, NULL);

    max = *n;
    capacity = 0;
    num = 0;
    res = NULL;
    while (num < max) {
	if (!readbuf(in, tarheader, 512)) {
	    g_error("could not read tar header");
	    return NULL;
	}

	if (iszero(tarheader, 512)) {
	    readbuf(in, tarheader, 512);
	    break;
	}
	
	if (!istarheader(tarheader)) {
	    g_error("invalid tar file");
	    return NULL;
	}
	mod = (-(gint)getoctnum(&tarheader[124], 12))&511;
	
	if (num >= capacity) {
	    capacity += 8;
	    res = (GsfSpriteGfx **)g_realloc(res,
					     capacity*sizeof(GsfSpriteGfx *));
	}
	res[num] = loadpng(in);
	if (!readbuf(in, tarheader, mod)) {
	    g_error("could not read tar file");
	    return NULL;
	}
	num++;
    }
    if (num != capacity)
	res = (GsfSpriteGfx **)g_realloc(res, num*sizeof(GsfSpriteGfx *));
    *n = num;

    return res;
}

