#include "pieceview.h"

static void pieceview_class_init (PieceViewClass *klass);
static void pieceview_init       (PieceView      *pieceview);
static void pieceview_destroy    (GtkObject      *object);
static gint pieceview_expose     (GtkWidget      *widget,
				  GdkEventExpose *event);

static GtkDrawingAreaClass *parent_class = NULL;

GtkType
pieceview_get_type(void)
{
    static GtkType pieceview_type = 0;

    if (!pieceview_type) {
        GtkTypeInfo pieceview_info = {
            "PieceView",
            sizeof(PieceView),
            sizeof(PieceViewClass),
            (GtkClassInitFunc) pieceview_class_init,
            (GtkObjectInitFunc) pieceview_init,
            (GtkArgSetFunc) NULL,
            (GtkArgGetFunc) NULL,
        };

        pieceview_type = gtk_type_unique(gtk_drawing_area_get_type(),
                                         &pieceview_info);
    }

    return pieceview_type;
}

static void
pieceview_class_init(PieceViewClass *klass)
{
    GtkObjectClass *object_class;
    GtkWidgetClass *widget_class;

    object_class = (GtkObjectClass*)klass;
    widget_class = (GtkWidgetClass*)klass;
    parent_class = gtk_type_class(gtk_drawing_area_get_type());

    widget_class->expose_event = pieceview_expose;
    object_class->destroy = pieceview_destroy;
}

static void
pieceview_init(PieceView *pieceview)
{
    pieceview->gfx = NULL;
}

GtkWidget *
pieceview_new(void)
{
    return gtk_type_new(pieceview_get_type());
}

void
pieceview_destroy(GtkObject *object)
{
    PieceView *pieceview;

    g_return_if_fail(object != NULL && IS_PIECEVIEW(object));

    pieceview = PIECEVIEW(object);

    pieceview_set(pieceview, NULL);

    (* GTK_OBJECT_CLASS(parent_class)->destroy)(object);
}

static void
pieceview_update(PieceView *view, GdkRectangle *area)
{
    GdkRectangle warea;
    GtkWidget *widget = GTK_WIDGET(view);
    
    if (!GTK_WIDGET_DRAWABLE(widget))
	return;
    
    if (!area) {
	gint tmpw, tmph;
	gdk_window_get_size(widget->window, &tmpw, &tmph);
	
	warea.x = 0;
	warea.y = 0;
	warea.width = tmpw;
	warea.height = tmph;
	area = &warea;
    }
    if (view->gfx) {
	GdkGC *gc;
	gint sx, sy, dx, dy, x, y, w, h;
	
	gdk_window_clear_area(widget->window,
			      area->x,
			      area->y,
			      area->width,
			      area->height);
	
	w = view->gfx->width;
	h = view->gfx->height;
	sx = 0;
	sy = 0;
	dx = x = (widget->allocation.width - w) / 2;
	dy = y = (widget->allocation.height - h) / 2;
	
	if (dx < area->x) {
	    sx = area->x - dx;
	    w -= sx;
	    dx = area->x;
	}
	if (w > (gint)area->width)
	    w = area->width;
	if (w <= 0)
	    return;
	
	if (dy < area->y) {
	    sy = area->y - dy;
	    h -= sy;
	    dy = area->y;
	}
	if (h > (gint)area->height)
	    h = area->height;
	if (h <= 0)
	    return;
	
	gc = gdk_gc_new(GTK_WIDGET(view)->window);
	gdk_gc_set_clip_mask(gc, view->gfx->mask);
	gdk_gc_set_clip_origin(gc, x, y);
	gdk_draw_pixmap(widget->window, gc, view->gfx->pixmap,
			sx, sy,
			dx, dy, view->gfx->width, view->gfx->height);
    }
    
}

void
pieceview_set(PieceView *pieceview, GsfSpriteGfx *gfx)
{
    g_return_if_fail(pieceview != NULL && IS_PIECEVIEW(pieceview));
    g_return_if_fail(gfx == NULL || GSF_IS_SPRITEGFX(gfx));

    if (gfx == pieceview->gfx)
	return;
    
    if (pieceview->gfx) {
	gsf_spritegfx_unref(pieceview->gfx);
	pieceview->gfx = NULL;
    }

    pieceview->gfx = gfx;
    if (pieceview->gfx) {
	gsf_spritegfx_ref(pieceview->gfx);
	gtk_object_sink(GTK_OBJECT(pieceview->gfx));
    }
    
    pieceview_update(pieceview, NULL);
}

static gint
pieceview_expose(GtkWidget      *widget,
		 GdkEventExpose *event)
{
    g_return_val_if_fail(widget != NULL && IS_PIECEVIEW(widget), FALSE);
    g_return_val_if_fail(event != NULL, FALSE);

    pieceview_update(PIECEVIEW(widget), &event->area);

    return FALSE;
}
