#include <glib.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <gsf/gsf.h>
#include <gdk_imlib.h>

#include <stdlib.h>

#include "tetrisboard.h"
#include "pieceview.h"

gboolean quitting = FALSE;

static void
quit(void)
{
    quitting = TRUE;
    gtk_main_quit();
}

static void
quit_press(GtkWidget *widget, gpointer data)
{
    if (data)
	gtk_widget_destroy(GTK_WIDGET(data));
}

static void
handle_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer data)
{
    TetrisBoard *tb;

    tb = TETRISBOARD(data);

    switch(event->keyval) {
    case GDK_KP_Up:    case GDK_KP_8:  tetris_board_rccw_pressed(tb);  break;
    case GDK_KP_Begin: case GDK_KP_5:  tetris_board_rcw_pressed(tb);   break;
    case GDK_KP_Left:  case GDK_KP_4:  tetris_board_left_pressed(tb);  break;
    case GDK_KP_Right: case GDK_KP_6:  tetris_board_right_pressed(tb); break;
    case GDK_KP_Down:  case GDK_KP_2:  tetris_board_down_pressed(tb);  break;
    case GDK_q:                                                        break;
    default:
        return /*FALSE*/;
    }

    return /*TRUE*/;
}

static void
handle_key_release_event(GtkWidget *widget, GdkEventKey *event, gpointer data)
{
    TetrisBoard *tb;

    tb = TETRISBOARD(data);
    switch(event->keyval) {
    case GDK_KP_Up:    case GDK_KP_8:  tetris_board_rccw_released(tb);  break;
    case GDK_KP_Begin: case GDK_KP_5:  tetris_board_rcw_released(tb);   break;
    case GDK_KP_Left:  case GDK_KP_4:  tetris_board_left_released(tb);  break;
    case GDK_KP_Right: case GDK_KP_6:  tetris_board_right_released(tb); break;
    case GDK_KP_Down:  case GDK_KP_2:  tetris_board_down_released(tb);  break;
    case GDK_q:                        quit_press(widget, widget);      break;
    default:
        return /*FALSE*/;
    }

    return /*TRUE*/;
}

static void
removed_lines(GtkWidget *widget, gint lines, gpointer data)
{
    GString *s = g_string_new(NULL);
    gchar *str;
    gint newsc;
    GtkWidget *w;

    w = GTK_WIDGET(data);

    gtk_label_get(GTK_LABEL(w), &str);
    newsc = atoi(str) + lines;
    g_string_sprintf(s, "%d", newsc);
    gtk_label_set(GTK_LABEL(w), s->str);
    g_string_free(s, TRUE);
}

static void
game_over(GtkWidget *widget, gpointer data)
{
}

static void
next_piece(GtkWidget *widget, GtkObject *piece, gpointer data)
{
    GsfPixmapSprite *ps;
    GtkWidget *w;
 
    ps = GSF_PIXMAPSPRITE(piece);
    w = GTK_WIDGET(data);

    pieceview_set(PIECEVIEW(w), ps->gfx[ps->currentframe]);
}

static void
resetting(GtkWidget *widget, gpointer data)
{
    GtkWidget *w;
    
    w = GTK_WIDGET(data);
    gtk_label_set(GTK_LABEL(w), "0");
}

static void
newgame(GtkWidget *widget, gpointer data)
{
    TetrisBoard *tb;

    tb = TETRISBOARD(data);
    tetris_board_reset(tb);
    tetris_board_start(tb);
}

static GtkWidget *
init_game(void)
{
    GtkWidget *window;
    GtkWidget *hbox;
    GtkWidget *vbox;
    GtkWidget *tetrisboard;
    GtkWidget *newgamebut;
    GtkWidget *quitbut;
    GtkWidget *nextpiecewidget;
    GtkWidget *scorewidget;

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);    
    gtk_container_border_width(GTK_CONTAINER(window), 10);

    hbox = gtk_hbox_new(FALSE, 10);
    gtk_container_border_width(GTK_CONTAINER(hbox), 10);

    tetrisboard = tetris_board_new(12, 24);
    gtk_box_pack_start(GTK_BOX(hbox), tetrisboard, TRUE, TRUE, 0);
    gtk_widget_show(tetrisboard);
    
    vbox = gtk_vbox_new(FALSE, 5);

    {
	GtkWidget *frame;
	frame = gtk_frame_new("Next Piece");
	nextpiecewidget = pieceview_new();
	gtk_drawing_area_size(GTK_DRAWING_AREA(nextpiecewidget), 40, 40);
	gtk_container_border_width(GTK_CONTAINER(frame), 10);
	gtk_container_add(GTK_CONTAINER(frame), nextpiecewidget);
	gtk_widget_show(nextpiecewidget);

	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 0);
	gtk_widget_show(frame);
    }
    
    {
	GtkWidget *frame;

	frame = gtk_frame_new("Score");
	
	scorewidget = gtk_label_new("0");
	gtk_container_border_width(GTK_CONTAINER(frame), 10);
	gtk_container_add(GTK_CONTAINER(frame), scorewidget);
	gtk_widget_show(scorewidget);

	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 0);
	gtk_widget_show(frame);
    }
    
    quitbut = gtk_button_new_with_label("Quit"),
    gtk_box_pack_end(GTK_BOX(vbox), quitbut, FALSE, TRUE, 0);
    gtk_widget_show(quitbut);

    newgamebut = gtk_button_new_with_label("New Game");
    gtk_box_pack_end(GTK_BOX(vbox), newgamebut, FALSE, TRUE, 0);
    gtk_widget_show(newgamebut);
    
    gtk_box_pack_end(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
    gtk_widget_show(vbox);

    gtk_container_add(GTK_CONTAINER(window), hbox);
    gtk_widget_show(hbox);
			
    gtk_signal_connect(GTK_OBJECT(window), "destroy",
		       GTK_SIGNAL_FUNC(quit), NULL);
    gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
		       GTK_SIGNAL_FUNC(handle_key_press_event), tetrisboard);
    gtk_signal_connect(GTK_OBJECT(window), "key_release_event",
		       GTK_SIGNAL_FUNC(handle_key_release_event), tetrisboard);
    gtk_widget_set_events(window,
			  gtk_widget_get_events(window)
			  | GDK_KEY_PRESS_MASK
			  | GDK_KEY_RELEASE_MASK);

    gtk_signal_connect(GTK_OBJECT(tetrisboard), "removed_lines",
		       GTK_SIGNAL_FUNC(removed_lines), scorewidget);
    gtk_signal_connect(GTK_OBJECT(tetrisboard), "game_over",
		       GTK_SIGNAL_FUNC(game_over), NULL);
    gtk_signal_connect(GTK_OBJECT(tetrisboard), "next_piece",
		       GTK_SIGNAL_FUNC(next_piece), nextpiecewidget);
    gtk_signal_connect(GTK_OBJECT(tetrisboard), "resetting",
		       GTK_SIGNAL_FUNC(resetting), scorewidget);
    

    gtk_signal_connect(GTK_OBJECT(quitbut), "clicked",
		       GTK_SIGNAL_FUNC(quit_press), window);
    gtk_signal_connect(GTK_OBJECT(newgamebut), "clicked",
		       GTK_SIGNAL_FUNC(newgame), tetrisboard);

    return window;
}

int
main(int argc, char *argv[])
{
    GtkWidget *game;

    gtk_init(&argc, &argv);
    gdk_imlib_init();
    gtk_widget_push_visual(gdk_imlib_get_visual());
    gtk_widget_push_colormap(gdk_imlib_get_colormap());
/*     gsf_spritegfx_set_collconvert(1, 10); */

    game = init_game();
    gtk_widget_show(game);
    
    gtk_main();

    gtk_type_describe_tree(gtk_object_get_type(), TRUE);

    return 0;
}
