/*
 * TetrisBoard
 *
 * $Id: tetrisboard.h,v 1.3 1998/07/30 01:27:49 torsk Exp $
 */
#ifndef TETRISBOARD_H
#define TETRISBOARD_H 1

#include <glib.h>
#include <gtk/gtk.h>
#include <gsf/gsf.h>

/* FIX: Commented out due to annoing Emacs indent:
#ifdef __cplusplus
extern "C" {
#endif
*/

#define TETRISBOARD(obj)             (GTK_CHECK_CAST (obj, tetris_board_get_type (), TetrisBoard))
#define TETRISBOARD_CLASS(klass)     (GTK_CHECK_CLASS_CAST (klass, tetris_board_get_type (), TetrisBoardClass))
#define IS_TETRISBOARD(obj)          (GTK_CHECK_TYPE (obj, tetris_board_get_type ()))

typedef struct _TetrisBoardClass TetrisBoardClass;
typedef struct _TetrisBoard      TetrisBoard;

typedef struct _GfxSeq           GfxSeq;

struct _GfxSeq
{
    gint num;
    GsfSpriteGfx **seq;
};

struct _TetrisBoard
{
    GsfSpriteView view;

    guint width, height;
    
    int numsprgfx;
    GfxSeq *sprgfx;

    GsfSpriteGfx *block;
    GsfPixmapSprite *board;
    GsfMovePmSprite *next;
    GsfMovePmSprite *piece;
    GsfPixmapSprite *gameover;
    gint fall;
    gint fallspeed;
    gint move;
    gint movespeed;

    gboolean rcw_pressed;
    gboolean rccw_pressed;
    gboolean left_pressed;
    gboolean right_pressed;
    gboolean down_pressed;
    
    gboolean rcw_released;
    gboolean rccw_released;
    gboolean left_released;
    gboolean right_released;
    gboolean down_released;

    guint timer;
};

struct _TetrisBoardClass
{
    GsfSpriteViewClass parent_class;
    
    void (* removed_lines)(TetrisBoard *tb, gint numlines);
    void (* game_over)    (TetrisBoard *tb);
    void (* next_piece)   (TetrisBoard *tb, GsfPixmapSprite *s);
    void (* resetting)    (TetrisBoard *tb);
};

GtkType           tetris_board_get_type       (void);
GtkWidget*        tetris_board_new            (guint width, guint height);

void              tetris_board_rcw_pressed    (TetrisBoard *tb);
void              tetris_board_rcw_released   (TetrisBoard *tb);
void              tetris_board_rccw_pressed   (TetrisBoard *tb);
void              tetris_board_rccw_released  (TetrisBoard *tb);
void              tetris_board_left_pressed   (TetrisBoard *tb);
void              tetris_board_left_released  (TetrisBoard *tb);
void              tetris_board_right_pressed  (TetrisBoard *tb);
void              tetris_board_right_released (TetrisBoard *tb);
void              tetris_board_down_pressed   (TetrisBoard *tb);
void              tetris_board_down_released  (TetrisBoard *tb);

void              tetris_board_reset          (TetrisBoard *tb);
void              tetris_board_start          (TetrisBoard *tb);
void              tetris_board_stop           (TetrisBoard *tb);

/* FIX: Commented out due to annoing Emacs indent:
#ifdef __cplusplus
}
#endif
*/  

#endif /* TETRISBOARD_H */
