/* GSF - GTK+ SpriteField.  Library to use sprites in GTK+
 * Copyright (C) 1998  Tore Skjellnes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "gsfclusterizer.h"
#include "gsfmemarr.h"

/* typedef struct */
/* { */
/*     gint left; */
/*     gint top; */
/*     gint right; */
/*     gint bottom; */
/* } GsfClusterizerRect; */



static void gsf_clusterizer_class_init (GsfClusterizerClass *klass);
static void gsf_clusterizer_init       (GsfClusterizer      *clusterizer);
static void gsf_clusterizer_destroy    (GtkObject           *object);


/* Local data */
static GtkObjectClass *parent_class = NULL;

/* static gboolean rect_ma_init = FALSE; */
/* static GsfMemArr rect_ma; */

GtkType
gsf_clusterizer_get_type(void)
{
    static GtkType clusterizer_type = 0;
    
    if (!clusterizer_type) {
	GtkTypeInfo clusterizer_info = {
	    "GsfClusterizer",
	    sizeof(GsfClusterizer),
	    sizeof(GsfClusterizerClass),
	    (GtkClassInitFunc) gsf_clusterizer_class_init,
	    (GtkObjectInitFunc) gsf_clusterizer_init,
	    (GtkArgSetFunc) NULL,
	    (GtkArgGetFunc) NULL,
	};

	clusterizer_type = gtk_type_unique(gtk_object_get_type(),
					   &clusterizer_info);
    }

    return clusterizer_type;
}

static void
gsf_clusterizer_class_init(GsfClusterizerClass *klass)
{
    GtkObjectClass *object_class;

    object_class = (GtkObjectClass*)klass;
    parent_class = gtk_type_class(gtk_object_get_type());    
    object_class->destroy = gsf_clusterizer_destroy;
    
/*     if (!rect_ma_init) { */
/* 	rect_ma_init = TRUE; */
/* 	GSF_MA_INIT(&rect_ma, GsfClusterizerRect); */
/*     } */
}

static void
gsf_clusterizer_init(GsfClusterizer *clusterizer)
{
    clusterizer->list = NULL;
}

GsfClusterizer *
gsf_clusterizer_new(void)
{
    return gtk_type_new(gsf_clusterizer_get_type());
}

void
gsf_clusterizer_ref(GsfClusterizer *c)
{
    g_return_if_fail(c != NULL && GSF_IS_CLUSTERIZER(c));
    gtk_object_ref(GTK_OBJECT(c));
}

void
gsf_clusterizer_unref(GsfClusterizer *c)
{
    g_return_if_fail(c != NULL && GSF_IS_CLUSTERIZER(c));
    gtk_object_unref(GTK_OBJECT(c));
}

     

static void
gsf_clusterizer_destroy(GtkObject *object)
{
    GSList *iter;
    GsfClusterizer *clusterizer;

    g_return_if_fail(object != NULL);
    g_return_if_fail(GSF_IS_CLUSTERIZER(object));

    clusterizer = GSF_CLUSTERIZER(object);
    
    iter = clusterizer->list;
    while (iter) {
	GSF_RECTANGLE_FREE(iter->data);
	iter->data = NULL;
	iter = iter->next;
    }

    g_slist_free(clusterizer->list);
    clusterizer->list = NULL;

    if (GTK_OBJECT_CLASS(parent_class)->destroy)
	(* GTK_OBJECT_CLASS(parent_class)->destroy)(object);
}

static gboolean
gsf_clusterizer_merge_ok(GsfRectangle *source1, GsfRectangle *source2)
{
    gint delta;

    delta = source2->x - source1->x;
    if (delta > (gint)source1->width || -delta > (gint)source2->width)
	return FALSE;
    
    delta = source2->y - source1->y;
    return (delta <= (gint)source1->height && -delta <= (gint)source2->height);
}

void
gsf_clusterizer_add(GsfClusterizer *clusterizer,
		    GsfRectangle *rect)
{
    GSList *tmp;
    GSList *prev;

    g_return_if_fail(clusterizer != NULL);
    g_return_if_fail(GSF_IS_CLUSTERIZER(clusterizer));
    
    /* Warn if rectangle empty (or invalid) */
    g_return_if_fail(rect != NULL && rect->width > 0 && rect->height > 0);

    /* FIX: too simple algorithm? Joins with 1. intersecting/adjacent rect. */
    prev = NULL;
    tmp = clusterizer->list;
    while (tmp) {
	GsfRectangle *cr = (GsfRectangle *)tmp->data;

	if (gsf_clusterizer_merge_ok(rect, cr)) {
	    /* make bounding box */
	    gsf_rectangle_makebound(rect, rect, cr);
	    
	    /* Detach from cluster list */
	    if (prev)
		prev->next = tmp->next;
	    else
		clusterizer->list = tmp->next;
	    GSF_RECTANGLE_FREE(cr);
	    g_slist_free_1(tmp);
	    
	    /* Try again */
	    prev = NULL;
	    tmp = clusterizer->list;
	} else {
	    prev = tmp;
	    tmp = tmp->next;
	}
    }	

    /* Insert rectangle */
    clusterizer->list = g_slist_prepend(clusterizer->list, rect);
}

void
gsf_clusterizer_add_point(GsfClusterizer *clusterizer, gint x, gint y)
{
    GsfRectangle *n = GSF_RECTANGLE_NEW();
    gsf_rectangle_set(n, x, y, 1, 1);
    gsf_clusterizer_add(clusterizer, n);
}

GsfClusterizerIx
gsf_clusterizer_first(GsfClusterizer *clusterizer)
{
    g_return_val_if_fail(clusterizer != NULL, NULL);
    g_return_val_if_fail(GSF_IS_CLUSTERIZER(clusterizer), NULL);
    
    return clusterizer->list;
}

GsfClusterizerIx
gsf_clusterizer_next(GsfClusterizer *clusterizer, GsfClusterizerIx ix)
{
    g_return_val_if_fail(clusterizer != NULL, NULL);
    g_return_val_if_fail(GSF_IS_CLUSTERIZER(clusterizer), NULL);
    
    return g_slist_next(ix);
}

GsfRectangle *
gsf_clusterizer_get(GsfClusterizer *clusterizer, GsfClusterizerIx ix)
{
    g_return_val_if_fail(clusterizer != NULL, NULL);
    g_return_val_if_fail(GSF_IS_CLUSTERIZER(clusterizer), NULL);
    g_return_val_if_fail(ix != NULL, NULL);

    return (GsfRectangle *)ix->data;
}

GsfRectangle *
gsf_clusterizer_pop(GsfClusterizer *clusterizer)
{
    GSList *n;
    GsfRectangle *r;
    
    g_return_val_if_fail(clusterizer != NULL, NULL);
    g_return_val_if_fail(GSF_IS_CLUSTERIZER(clusterizer), NULL);

    n = clusterizer->list;
    if (!n)
	return NULL;
    
    clusterizer->list = n->next;
    r = (GsfRectangle *)n->data;
    g_slist_free_1(n);

    return r;
}
