/* GSF - GTK+ SpriteField.  Library to use sprites in GTK+
 * Copyright (C) 1998  Tore Skjellnes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "gsfcollisionmask.h"

#include <stdlib.h>

static void gsf_collisionmask_class_init(GsfCollisionMaskClass *klass);
static void gsf_collisionmask_init      (GsfCollisionMask      *pixmap);
static void gsf_collisionmask_destroy   (GtkObject             *object);

static GtkObjectClass *parent_class = NULL;

GtkType
gsf_collisionmask_get_type(void)
{
    static GtkType collisionmask_type = 0;

    if (!collisionmask_type) {
	GtkTypeInfo collisionmask_info = {
	    "GsfCollisionMask",
	    sizeof(GsfCollisionMask),
	    sizeof(GsfCollisionMaskClass),
	    (GtkClassInitFunc) gsf_collisionmask_class_init,
	    (GtkObjectInitFunc) gsf_collisionmask_init,
	    (GtkArgSetFunc) NULL,
	    (GtkArgGetFunc) NULL,
	};

	collisionmask_type = gtk_type_unique(gtk_object_get_type(),
					     &collisionmask_info);
    }

    return collisionmask_type;
}

static void
gsf_collisionmask_class_init(GsfCollisionMaskClass *klass)
{
    GtkObjectClass *object_class;

    object_class = (GtkObjectClass*)klass;
    parent_class = gtk_type_class(gtk_object_get_type());

    object_class->destroy = gsf_collisionmask_destroy;
}

static void
gsf_collisionmask_init(GsfCollisionMask *collisionmask)
{
    collisionmask->width = 0;
    collisionmask->wpl = 0;
    collisionmask->height = 0;
    collisionmask->data = NULL;
}

GsfCollisionMask *
gsf_collisionmask_new(void)
{
    return gtk_type_new(gsf_collisionmask_get_type());
}

void
gsf_collisionmask_ref(GsfCollisionMask *mask)
{
    g_return_if_fail(mask != NULL && GSF_IS_COLLISIONMASK(mask));
    gtk_object_ref(GTK_OBJECT(mask));
}

void
gsf_collisionmask_unref(GsfCollisionMask *mask)
{
    g_return_if_fail(mask != NULL && GSF_IS_COLLISIONMASK(mask));
    gtk_object_unref(GTK_OBJECT(mask));
}

static void
gsf_collisionmask_destroy(GtkObject *object)
{
    GsfCollisionMask *collisionmask;
    g_return_if_fail(object != NULL && GSF_IS_COLLISIONMASK(object));

    collisionmask = GSF_COLLISIONMASK(object);

    if (collisionmask->data) {
	g_free(collisionmask->data);
	collisionmask->width = 0;
	collisionmask->wpl = 0;
	collisionmask->height = 0;
	collisionmask->data = NULL;
    }
    
    if (GTK_OBJECT_CLASS(parent_class)->destroy)
	(* GTK_OBJECT_CLASS(parent_class)->destroy)(object);
}

static guchar bitflip[256] = {
    0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0,
    0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0,
    0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8,
    0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8,
    0x04, 0x84, 0x44, 0xc4, 0x24, 0xa4, 0x64, 0xe4,
    0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4,
    0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec,
    0x1c, 0x9c, 0x5c, 0xdc, 0x3c, 0xbc, 0x7c, 0xfc,
    0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2,
    0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2,
    0x0a, 0x8a, 0x4a, 0xca, 0x2a, 0xaa, 0x6a, 0xea,
    0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa,
    0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6,
    0x16, 0x96, 0x56, 0xd6, 0x36, 0xb6, 0x76, 0xf6,
    0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee,
    0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe,
    0x01, 0x81, 0x41, 0xc1, 0x21, 0xa1, 0x61, 0xe1,
    0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1,
    0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9,
    0x19, 0x99, 0x59, 0xd9, 0x39, 0xb9, 0x79, 0xf9,
    0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5,
    0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5,
    0x0d, 0x8d, 0x4d, 0xcd, 0x2d, 0xad, 0x6d, 0xed,
    0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd,
    0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3,
    0x13, 0x93, 0x53, 0xd3, 0x33, 0xb3, 0x73, 0xf3,
    0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb,
    0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb,
    0x07, 0x87, 0x47, 0xc7, 0x27, 0xa7, 0x67, 0xe7,
    0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7,
    0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef,
    0x1f, 0x9f, 0x5f, 0xdf, 0x3f, 0xbf, 0x7f, 0xff
};


/* FIX: image bit order is always assumed to be Whatever Endian :-)
 * (no nice way to find bitmap_bit_order in gdk_image ?)
 * I have to swap the bits to make it work.
 *
 * The collision map bits is supposed to look like this when only the
 * first 10 bits is set in the mask:
 *
 *   MSBit                                    LSBit
 *    1111 1111  1100 0000  0000 0000  0000 0000
 *  x=0123 4567  89...
 *
 * The byte endianness of the produced mask is the native one.
 */
static void
gsf_collisionmask_render_coll_mask(GsfCollisionMask *collisionmask,
				   GdkImage *maskimage)
{
    guint wpl;
    guint rot;
    guint srcmod;
    guint32 tmp;
    guint32 padmask;
    guchar *src;
    guint32 *dest;
    guint32 *destend;
    guint32 *destlend;
    if (maskimage->depth != 1 || maskimage->bpp != 1)
	g_warning("Mask image has invalid depth (depth = %d, bpp = %d)",
		  maskimage->depth, maskimage->bpp);

    /* Wasting some memory in the name of code simplicity (i.e.
       lazyness) */
    wpl = ((maskimage->width+31)>>5) + 1;
    rot = (-maskimage->width) & 0x1F;
    
    padmask = 0xFFFFFFFFul << rot;

    if (collisionmask->data)
	g_free(collisionmask->data);

    collisionmask->width = maskimage->width;
    collisionmask->wpl = wpl;
    collisionmask->height = maskimage->height;
    collisionmask->data = g_new(guint32, wpl*maskimage->height);

    src = (guchar *)maskimage->mem;
    srcmod = maskimage->bpl - 4*(wpl - 2);
    dest = collisionmask->data;
    destend = dest + wpl*maskimage->height;
    while (dest < destend) {
	destlend = dest + wpl - 2;
	while (dest < destlend) {
	    *dest++ = ((bitflip[src[0]]<<24)
		       + (bitflip[src[1]]<<16)
		       + (bitflip[src[2]]<<8)
		       + bitflip[src[3]]);
	    src += 4;
	}
	tmp = bitflip[src[0]]<<24;
	if (srcmod > 1) { /* Be sure we are allowed to read byte... */
	    tmp |= bitflip[src[1]]<<16;
	    if (srcmod > 2) {
		tmp |= bitflip[src[2]]<<8;
		if (srcmod > 3)
		    tmp |= bitflip[src[3]];
	    }
	}
	*dest++ = tmp & padmask;
	*dest++ = 0; /* Zero out collision pad word */
	src += srcmod;
    }
}

void
gsf_collisionmask_set_coll(GsfCollisionMask *collisionmask,
			   GdkImage *collmask)
{
    g_return_if_fail(collisionmask != NULL
		     && GSF_IS_COLLISIONMASK(collisionmask));

    gsf_collisionmask_render_coll_mask(collisionmask, collmask);
}

void
gsf_collisionmask_set(GsfCollisionMask *collisionmask,
		      GdkBitmap *bitmap)
{
    GdkImage *maskimage;
    gint maskwidth, maskheight;
    
    g_return_if_fail(collisionmask != NULL
		     && GSF_IS_COLLISIONMASK(collisionmask));
    g_return_if_fail(bitmap != NULL);
    
    gdk_window_get_size(bitmap, &maskwidth, &maskheight);
    maskimage = gdk_image_get(bitmap, 0, 0, maskwidth, maskheight);
    g_return_if_fail(maskimage != NULL);
    gsf_collisionmask_set_coll(collisionmask, maskimage);
    gdk_image_destroy(maskimage);
}

gboolean
gsf_collisionmask_collides(GsfCollisionMask *collisionmask1,
			   gint x1,
			   gint y1,
			   GsfCollisionMask *collisionmask2,
			   gint x2,
			   gint y2)
{
    guint32 *data1;
    guint32 *data2;
    guint32 *end2;
    gint w, h;
    guint wwpl, rot, nrot, mod1, mod2;
    gint deltax;
    gint deltay;
    
    g_return_val_if_fail(collisionmask1 != NULL
			 && GSF_IS_COLLISIONMASK(collisionmask1)
			 && collisionmask1->data != NULL,
			 FALSE);
    g_return_val_if_fail(collisionmask2 != NULL
			 && GSF_IS_COLLISIONMASK(collisionmask2)
			 && collisionmask2->data != NULL,
			 FALSE);

    deltax = x2 - x1;
    deltay = y2 - y1;
    
    if (deltax < 0) {
	GsfCollisionMask *tmp;

	tmp = collisionmask1;
	collisionmask1 = collisionmask2;
	collisionmask2 = tmp;
	deltax = -deltax;
	deltay = -deltay;
    }
    
    w = (gint)collisionmask1->width - deltax;
    if (w > (gint)collisionmask2->width)
	w = collisionmask2->width;

    /* Return if rectangles do not intersect */
    if (w <= 0)
	return FALSE;

    data1 = collisionmask1->data;
    data2 = collisionmask2->data;

    if (deltay < 0) {
	h = (gint)collisionmask2->height + deltay;
	if (h > (gint)collisionmask1->height)
	    h = collisionmask1->height;
	
	data2 -= deltay*collisionmask2->wpl;
    } else {
	h = (gint)collisionmask1->height - deltay;
	if (h > (gint)collisionmask2->height)
	    h = collisionmask2->height;
	
	data1 += deltay*collisionmask1->wpl;
    }

    /* Return if rectangles do not intersect */
    if (h <= 0)
	return FALSE;

    wwpl = ((w + 31) >> 5);
    end2 = data2 + h*collisionmask2->wpl;
    data1 += deltax>>5;
    rot = deltax & 31;
    nrot = 32 - rot;
    mod2 = collisionmask2->wpl - wwpl;
    mod1 = collisionmask1->wpl - wwpl;

    while (data2 < end2) {
	guint32 *lend2;

	lend2 = data2 + wwpl;
	if (rot == 0) {
	    /* Special case for no rot ((x >> 32) == x on i386!!) */
	    while (data2 < lend2) {
		if ( (*data2 & *data1) )
		    return TRUE;
		data2++;
		data1++;
	    }
	} else {
	    guint32 a;
	    a = *data1;
	    while (data2 < lend2) {
		guint32 d = *data2;

		if (d & (a << rot))
		    return TRUE;
		a = *++data1;
		if (d & (a >> nrot))
		    return TRUE;

 		/* Check if collision */
/* 		if ( (*data2 & ((a << rot) | (b >> nrot))) )
		    return TRUE;
*/
		data2++;
	    }
	}
	data1 += mod1;
	data2 += mod2;
    }

    return FALSE;
}

gboolean
gsf_collisionmask_collides_at(GsfCollisionMask *collisionmask,
			      gint x,
			      gint y)
{
    g_return_val_if_fail(collisionmask != NULL
			 && GSF_IS_COLLISIONMASK(collisionmask)
			 && collisionmask->data != NULL,
			 FALSE);

    /* Return false if point is outside mask */
    if ((guint)x >= (guint)collisionmask->width
	|| (guint)y >= (guint)collisionmask->height)
	return FALSE;

    /* Return bit at position (x,y) of mask */
    return (collisionmask->data[y*collisionmask->wpl + (x>>5)]
	    & (0x80000000ul >> (x & 0x1F))) != 0;
}



/* FIX: speed up this... */
void
gsf_collisionmask_scale(GsfCollisionMask *dest,
			GsfCollisionMask *source,
			guint mul, guint div)
{
    gboolean hit;
    guint nw;
    guint nwpl;
    guint nh;
    guint32 *tmp;
    guint32 *nd;
    guint x, y, x2, y2, sx, sy, ex, ey;

    g_return_if_fail(source != NULL
		     && GSF_IS_COLLISIONMASK(source)
		     && source->data != NULL);
    g_return_if_fail(dest != NULL
		     && GSF_IS_COLLISIONMASK(dest));

    if (mul <= 0)
	mul = 1;
    if (div <= 0)
	div = 1;

    /* Free dest data before scaling if possible */
    if (dest->data && dest->data != source->data) {
	g_free(dest->data);
	dest->data = NULL;
    }
    
    nw = (source->width + div - 1) * mul / div;
    nh = (source->height + div - 1) * mul / div;
    nwpl = ((nw + 31) >> 5) + 1;
    nd = g_new0(guint32, nwpl*nh);
    for (y = 0; y < nh; y++) {
	sy = y * div / mul;
	ey = ((y+1) * div - 1) / mul;
	if (ey >= source->height)
	    ey = source->height - 1;
	for (x = 0; x < nw; x++) {
	    sx = x * div / mul;
	    ex = ((x+1) * div - 1) / mul;
	    if (ex >= source->width)
		ex = source->width - 1;

	    hit = FALSE;
	    for (y2 = sy; y2 <= ey; y2++) {
		tmp = &source->data[y2*source->wpl];

		for (x2 = sx; x2 <= ex; x2++) {
		    if (tmp[x2 >> 5] & (0x80000000ul >> (x2 & 0x1F))) {
			hit = TRUE;
			break;
		    }
		}
		
		if (hit)
		    break;
	    }
	    
	    if (hit)
		nd[y*nwpl + (x >> 5)] |= (0x80000000ul >> (x & 0x1F));
	}
    }

    /* If dest data not freed above, free it now */
    if (dest->data)
	g_free(dest->data);
    
    dest->width = nw;
    dest->wpl = nwpl;
    dest->height = nh;
    dest->data = nd;
}


#define BITBLT_COPY 0
#define BITBLT_OR   1
#define BITBLT_AND  2
#define BITBLT_XOR  3

static void
gsf_collisionmask_bitblt(GsfCollisionMask *dest,
			 GsfCollisionMask *source,
			 gint xsrc,
			 gint ysrc,
			 gint xdest,
			 gint ydest,
			 guint width,
			 guint height,
			 gint operation)
{
    g_return_if_fail(source != NULL
		     && GSF_IS_COLLISIONMASK(dest)
		     && source->data != NULL);
    g_return_if_fail(dest != NULL
		     && GSF_IS_COLLISIONMASK(dest)
		     && dest->data != NULL);

    /* Clip */
    if ((guint)xsrc > (guint)source->width) {
	if ((gint)xsrc > (gint)source->width) return;
	xdest -= xsrc;
	width += xsrc;
	xsrc = 0;
    }
    if ((guint)xdest > (guint)dest->width) {
	if ((gint)xdest > (gint)dest->width) return;
	xsrc -= xdest;
	width += xdest;
	xdest = 0;
    }
    if ((gint)width <= 0) return;
    if (width > source->width - xsrc) width = source->width - xsrc;
    if (width > dest->width - xdest)  width = dest->width - xdest;
    if ((gint)width <= 0) return;

    if ((guint)ysrc > (guint)source->height) {
	if ((gint)ysrc > (gint)source->height) return;
	ydest -= ysrc;
	height += ysrc;
	ysrc = 0;
    }
    if ((guint)ydest > (guint)dest->height) {
	if ((gint)ydest > (gint)dest->height) return;
	ysrc -= ydest;
	height += ydest;
	ydest = 0;
    }
    if ((gint)height <= 0) return;
    if (height > source->height - ysrc) height = source->height - ysrc;
    if (height > dest->height - ydest)  height = dest->height - ydest;
    if ((gint)height <= 0) return;

    /* Do the operation */
    {
	guint32 *srcdata;
	guint32 *srcend;
	gint     srcmod;
	guint32 *destdata;
	guint32 *destendl;
	gint     destmod;

	guint32 *tmp;
	guint32 *srcbuf;
	guint32 *srcp;
	guint32 *srcbufend;
	guint32  a, b;
	
	gint     srcrot = xsrc & 31;
	gint     srcnrot = 32 - srcrot;
	guint32  srcpadend = 0xFFFFFFFFul << ((-width) & 31);
	guint    swpl = (width + 31)>>5;
	
	gint     destrot = xdest & 31;
	gint     destnrot = 32 - destrot;
	guint32  padstart = 0xFFFFFFFFul >> (xdest & 31);
	guint32  padend = 0xFFFFFFFFul << ((-xdest - width) & 31);
	guint    dnum = ((xdest + width - 1) >> 5) - ((xdest) >> 5);

	gboolean overlapping = ((source == dest)
				&& (((ysrc < ydest
				      && ysrc + height - 1 >= ydest)
				     || (ysrc == ydest && xsrc < xdest))
				    && (xsrc + width - 1 >= xdest)));

	/* extra word to avoid reading out of bounds */
	srcbuf = g_new(guint32, swpl + 1);
	srcbufend = srcbuf + swpl - 1;

	if (overlapping) {
	    srcend = source->data + ysrc*source->wpl + (xsrc >> 5);
	    srcdata = srcend + (height-1)*source->wpl;
	    srcmod = -source->wpl;
	    destdata = dest->data + (ydest+height-1)*dest->wpl + (xdest >> 5);
	    destmod = -dest->wpl;
	} else {
	    srcdata = source->data + ysrc*source->wpl + (xsrc >> 5);
	    srcend = srcdata + (height-1)*source->wpl;
	    srcmod = source->wpl;
	    destdata = dest->data + ydest*dest->wpl + (xdest >> 5);
	    destmod = dest->wpl;
	}

	if (dnum == 0) {
	    padstart &= padend;
	    padend = 0;
	}

	while (srcdata != srcend) {
	    tmp = srcdata;
	    srcp = srcbuf;
	    
	    if (!srcrot) {
		while (srcp < srcbufend)
		    *srcp++ = *tmp++;
		*srcp = *tmp & srcpadend;
	    } else {
		b = 0;
		a = *tmp++;
		while (srcp < srcbufend) {
		    b = *tmp++;
		    *srcp++ = ((a << srcrot) | (b >> srcnrot));
		    a = b;
		}
		/* tmp not overrun because of extra word */
		*srcp = ((a << srcrot) | (*tmp >> srcnrot)) & srcpadend;
	    }

	    tmp = destdata;
	    srcp = srcbuf;

	    if (operation == BITBLT_COPY) {
		if (!destrot) {
		    *tmp = (*tmp & ~padstart) | (*srcp++ & padstart);
		    if (padend) {
			tmp++;
			while (srcp < srcbufend)
			    *tmp = *srcp++;
			*tmp = (*tmp & ~padend) | (*srcp & padend);
		    }
		} else {
		    destendl = tmp + dnum;
		    b = *srcp++;
		    *tmp = (*tmp & ~padstart) | ((b >> destrot) & padstart);
		    if (padend) {
			tmp++;
			while (tmp < destendl) {
			    a = *srcp++;
			    *tmp++ = ((a >> destrot) | (b << destnrot));
			    b = a;
			}
			a = *srcp;
			*tmp = ((*tmp & ~padend)
				| (((a >> destrot)|(b << destnrot)) & padend));
		    }
		}
	    } else if (operation == BITBLT_OR) {
		if (!destrot) {
		    *tmp |= (*srcp++ & padstart);
		    if (padend) {
			tmp++;
			while (srcp < srcbufend)
			    *tmp |= *srcp++;
			*tmp |= (*srcp & padend);
		    }
		} else {
		    destendl = tmp + dnum;
		    b = *srcp++;
		    *tmp |= ((b >> destrot) & padstart);
		    if (padend) {
			tmp++;
			while (tmp < destendl) {
			    a = *srcp++;
			    *tmp |= (a >> destrot);
			    *tmp++ |= (b << destnrot);
			    b = a;			    
			}
			a = *srcp;
			*tmp |= ((a >> destrot)|(b << destnrot)) & padend;
		    }
		}
	    } else if (operation == BITBLT_AND) {
		if (!destrot) {
		    *tmp &= ~padstart | (*srcp++ & padstart);
		    if (padend) {
			tmp++;
			while (srcp < srcbufend)
			    *tmp &= *srcp++;
			*tmp &= ~padend | (*srcp & padend);
		    }
		} else {
		    destendl = tmp + dnum;
		    b = *srcp++;
		    *tmp &= ~padstart | ((b >> destrot) & padstart);
		    if (padend) {
			tmp++;
			while (tmp < destendl) {
			    a = *srcp++;
			    *tmp++ &= (a >> destrot) | (b << destnrot);
			    b = a;
			}
			a = *srcp;
			*tmp &= (~padend
				 | (((a >> destrot)|(b << destnrot))&padend));
		    }
		}
	    } else if (operation == BITBLT_XOR) {
		if (!destrot) {
		    *tmp ^= (*srcp++ & padstart);
		    if (padend) {
			tmp++;
			while (srcp < srcbufend)
			    *tmp ^= *srcp++;
			*tmp ^= (*srcp & padend);
		    }
		} else {
		    destendl = tmp + dnum;
		    b = *srcp++;
		    *tmp ^= ((b >> destrot) & padstart);
		    if (padend) {
			tmp++;
			while (tmp < destendl) {
			    a = *srcp++;
			    *tmp ^= (a >> destrot);
			    *tmp++ ^= (b << destnrot);
			    b = a;
			}
			a = *srcp;
			*tmp ^= (((a >> destrot)|(b << destnrot)) & padend);
		    }
		}
	    } else {
		g_error("Invalid parameter `operation' in bitblt");
	    }
	    
	    destdata += destmod;
	    srcdata += srcmod;
	}
	
	g_free(srcbuf);
    }
}


#define UBITBLT_SET    0
#define UBITBLT_CLEAR  1
#define UBITBLT_INVERT 2

static void
gsf_collisionmask_ubitblt(GsfCollisionMask *dest,
			  gint xdest,
			  gint ydest,
			  guint width,
			  guint height,
			  gint operation)
{
    g_return_if_fail(dest != NULL
		     && GSF_IS_COLLISIONMASK(dest)
		     && dest->data != NULL);

    /* Clip */
    if ((guint)xdest > (guint)dest->width) {
	if ((gint)xdest > (gint)dest->width) return;
	width += xdest;
	xdest = 0;
    }
    if ((gint)width <= 0) return;
    if (width > dest->width - xdest)
	width = dest->width - xdest;

    if ((guint)ydest > (guint)dest->height) {
	if ((gint)ydest > (gint)dest->height) return;
	height += ydest;
	ydest = 0;
    }
    if ((gint)height <= 0) return;
    if (height > dest->height - ydest)
	height = dest->height - ydest;

    /* Do the `set' operation */
    {
	guint32 *destdata = dest->data + ydest*dest->wpl + (xdest >> 5);
	guint32 *destend = destdata + height*dest->wpl;
	
	guint32  padstart = 0xFFFFFFFFul >> (xdest & 31);
	gint     mid = ((xdest + width - 1) >> 5) - (xdest >> 5);
	guint32  padend = 0xFFFFFFFFul << ((-xdest - width) & 31);
	gint     mod;
	
	if (mid <= 0) {
	    padstart &= padend;
	    padend = 0;
	    mid = 1;
	}

	mod = dest->wpl - mid;
	if (operation == UBITBLT_SET) {
	    while (destdata < destend) {
		guint32 *e = destdata + mid;
		*destdata++ |= padstart;
		while (destdata < e)
		    *destdata++ = 0xFFFFFFFFul;
		*destdata |= padend;
		destdata += mod;
	    }
	} else if (operation == UBITBLT_CLEAR) {
	    padstart = ~padstart;
	    padend = ~padend;
	    while (destdata < destend) {
		guint32 *e = destdata + mid;
		*destdata++ &= padstart;
		while (destdata < e)
		    *destdata++ = 0;
		*destdata &= padend;
		destdata += mod;
	    }
	} else if (operation == UBITBLT_SET) {
	    while (destdata < destend) {
		guint32 *e = destdata + mid;
		*destdata++ ^= padstart;
		while (destdata < e)
		    *destdata++ ^= 0xFFFFFFFFul;
		*destdata ^= padend;
		destdata += mod;
	    }
	} else {
	    g_error("Invalid parameter `operation' in ubitblt");
	}
    }
}


void
gsf_collisionmask_or(GsfCollisionMask *dest,
		     GsfCollisionMask *source,
		     gint xsrc,
		     gint ysrc,
		     gint xdest,
		     gint ydest,
		     guint width,
		     guint height)
{
    gsf_collisionmask_bitblt(dest, source,
			     xsrc, ysrc, xdest, ydest, width, height,
			     BITBLT_OR);
}


void
gsf_collisionmask_copy(GsfCollisionMask *dest,
		       GsfCollisionMask *source,
		       gint xsrc,
		       gint ysrc,
		       gint xdest,
		       gint ydest,
		       guint width,
		       guint height)
{
    gsf_collisionmask_bitblt(dest, source,
			     xsrc, ysrc, xdest, ydest, width, height,
			     BITBLT_COPY);
}



void
gsf_collisionmask_setbit(GsfCollisionMask *dest,
			 gint xdest,
			 gint ydest,
			 guint width,
			 guint height)
{
    gsf_collisionmask_ubitblt(dest,
			      xdest, ydest, width, height,
			      UBITBLT_SET);
}

void
gsf_collisionmask_clearbit(GsfCollisionMask *dest,
			   gint xdest,
			   gint ydest,
			   guint width,
			   guint height)
{
    gsf_collisionmask_ubitblt(dest,
			      xdest, ydest, width, height,
			      UBITBLT_CLEAR);
}



#if 0
void
gsf_collisionmask_or(GsfCollisionMask *dest,
		     GsfCollisionMask *source,
		     gint xsrc,
		     gint ysrc,
		     gint xdest,
		     gint ydest,
		     guint width,
		     guint height)
{
    guint32 *sd;
    
    g_return_if_fail(source != NULL
		     && GSF_IS_COLLISIONMASK(dest)
		     && source->data != NULL);
    g_return_if_fail(dest != NULL
		     && GSF_IS_COLLISIONMASK(dest)
		     && dest->data != NULL);

    /* Clip */
    if ((guint)xsrc > (guint)source->width) {
	if ((gint)xsrc > (gint)source->width) return;
	xdest -= xsrc;
	width += xsrc;
	xsrc = 0;
    }
    if ((guint)xdest > (guint)dest->width) {
	if ((gint)xdest > (gint)dest->width) return;
	xsrc -= xdest;
	width += xdest;
	xdest = 0;
    }
    if ((gint)width <= 0) return;
    if (width > source->width - xsrc) width = source->width - xsrc;
    if (width > dest->width - xdest)  width = dest->width - xdest;
    if ((gint)width <= 0) return;

    if ((guint)ysrc > (guint)source->height) {
	if ((gint)ysrc > (gint)source->height) return;
	ydest -= ysrc;
	height += ysrc;
	ysrc = 0;
    }
    if ((guint)ydest > (guint)dest->height) {
	if ((gint)ydest > (gint)dest->height) return;
	ysrc -= ydest;
	height += ydest;
	ydest = 0;
    }
    if ((gint)height <= 0) return;
    if (height > source->height - ysrc) height = source->height - ysrc;
    if (height > dest->height - ydest)  height = dest->height - ydest;
    if ((gint)height <= 0) return;

/*     g_message("pastegfx: %d, %d - %d, %d - %d x %d", */
/* 	      xsrc, ysrc, xdest, ydest, width, height); */
    /* Do the `or' operation, could be faster by using just one rotation */


    /* Special case: moving data inside a spritegfx */
    if ((source == dest)
	&& (((ysrc < ydest && ysrc + height - 1 >= ydest)
	     || (ysrc == ydest && xsrc < xdest))
	    && (xsrc + width - 1 >= xdest))) {
	/* Overlapping, make a copy. Not very effective, but it works... */
	sd = g_new(guint32, height*source->wpl);
	memcpy(sd, source->data + ysrc*source->wpl, height*source->wpl*4);
    } else
	sd = source->data + ysrc*source->wpl;

    {
	guint32 *srcdata = sd + (xsrc >> 5);
	guint32 *srcend = sd + height*source->wpl;
	guint32 *destdata = dest->data + ydest*dest->wpl + (xdest >> 5);

	guint32  padstart = 0xFFFFFFFFul >> (xsrc & 31);
	gint     smid = ((xsrc + width - 1) >> 5) - (xsrc >> 5);
	guint32  padend = 0xFFFFFFFFul << ((-xsrc - width) & 31);
	gint     srcrot = xsrc & 31;
	gint     srcnrot = 32 - srcrot;

	gint     destrot = xdest & 31;
	gint     destnrot = 32 - destrot;

	gboolean extradest = (((xsrc+width-1) & 31) - srcrot + destrot) > 31;

	guint32 *destnextrow;
	guint32 *send2;
	guint32  a, b, s, da, db;

	if (smid == 0) {
	    padstart &= padend;
	    padend = 0;
	    smid = 1; /* Make the srcdata modulo correct */
	}

	while (srcdata < srcend) {
	    destnextrow = destdata + dest->wpl;
	    send2 = srcdata + smid;
	    
	    b = 0;
	    db = 0;

	    /* first source data word */
	    a = *srcdata++ & padstart;

	    /* middle data */
	    while (srcdata < send2) {
		b = *srcdata++;
		s = srcrot ? ((a << srcrot) | (b >> srcnrot)) : a;
		if (destrot) {
		    da = (s >> destrot) | db;
		    db = s << destnrot;
		} else
		    da = s;
		
		*destdata++ |= da;
		a = b;
	    }
	    
	    /* last data source word */
	    
	    /* srcdata not overrun because of extra word at end of line */
	    b = *srcdata & padend; 
	    s = srcrot ? ((a << srcrot) | (b >> srcnrot)) : a;
	    if (destrot) {
		da = (s >> destrot) | db;
		db = s << destnrot;
	    } else
		da = s;
	    *destdata |= da;

	    /* last data dest word */
	    s = b << srcrot;
	    if (destrot) {
		*++destdata |= (s >> destrot) | db;		
		if (extradest)
		    *++destdata |= s << destnrot;
	    } else
		*++destdata |= s;
	    
	    srcdata += source->wpl - smid;
	    destdata = destnextrow;
	}
    }
    
    if (sd != source->data + ysrc*source->wpl)
	g_free(sd);
}

#endif
