/* GSF - GTK+ SpriteField.  Library to use sprites in GTK+
 * Copyright (C) 1998  Tore Skjellnes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
/*
 * Gsf sprite graphics
 *
 * $Id: gsfcollisionmask.h,v 1.3 1998/07/30 03:35:40 torsk Exp $
 */
#ifndef __GSF_COLLISIONMASK_H__
#define __GSF_COLLISIONMASK_H__ 1

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glib.h>

/* FIX: Commented out due to annoing Emacs indent:
#ifdef __cplusplus
extern "C" {
#endif
*/

#define GSF_COLLISIONMASK(obj)             (GTK_CHECK_CAST (obj, gsf_collisionmask_get_type (), GsfCollisionMask))
#define GSF_COLLISIONMASK_CLASS(klass)     (GTK_CHECK_CLASS_CAST (klass, gsf_collisionmask_get_type (), GsfCollisionMaskClass))
#define GSF_IS_COLLISIONMASK(obj)          (GTK_CHECK_TYPE (obj, gsf_collisionmask_get_type ()))

typedef struct _GsfCollisionMaskClass GsfCollisionMaskClass;
typedef struct _GsfCollisionMask      GsfCollisionMask;

struct _GsfCollisionMask
{
    GtkObject object;

    guint width;
    guint wpl; /* 32bits words per line + pad = ((width+31)>>5)+1 */
    guint height;
    guint32 *data; /* Lines padded with 0's to get a 32 bits alignment */
};

struct _GsfCollisionMaskClass
{
    GtkObjectClass parent_class;
};

GtkType           gsf_collisionmask_get_type   (void);
GsfCollisionMask* gsf_collisionmask_new        (void);
void              gsf_collisionmask_ref        (GsfCollisionMask *mask);
void              gsf_collisionmask_unref      (GsfCollisionMask *mask);

void              gsf_collisionmask_set        (GsfCollisionMask *mask,
						GdkBitmap *bitmap);
void              gsf_collisionmask_set_coll   (GsfCollisionMask *mask,
						GdkImage *collmask);
gboolean          gsf_collisionmask_collides   (GsfCollisionMask *mask1,
						gint x1,
						gint y1,
						GsfCollisionMask *mask2,
						gint x2,
						gint y2);
gboolean          gsf_collisionmask_collides_at(GsfCollisionMask *mask,
						gint x,
						gint y);
void              gsf_collisionmask_scale      (GsfCollisionMask *dest,
						GsfCollisionMask *source,
						guint mul, guint div);
void              gsf_collisionmask_or         (GsfCollisionMask *dest,
						GsfCollisionMask *source,
						gint xsrc,
						gint ysrc,
						gint xdest,
						gint ydest,
						guint width,
						guint height);
void              gsf_collisionmask_copy       (GsfCollisionMask *dest,
						GsfCollisionMask *source,
						gint xsrc,
						gint ysrc,
						gint xdest,
						gint ydest,
						guint width,
						guint height);
void              gsf_collisionmask_setbit     (GsfCollisionMask *dest,
						gint xdest,
						gint ydest,
						guint width,
						guint height);
void              gsf_collisionmask_clearbit   (GsfCollisionMask *dest,
						gint xdest,
						gint ydest,
						guint width,
						guint height);

/* FIX: Commented out due to annoing Emacs indent:
#ifdef __cplusplus
}
#endif
*/  

#endif /* __GSF_COLLISIONMASK_H__ */
