/* GSF - GTK+ SpriteField.  Library to use sprites in GTK+
 * Copyright (C) 1998  Tore Skjellnes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
/*
 * Memory array allocator
 *
 * $Id: gsfmemarr.h,v 1.3 1998/07/30 03:35:41 torsk Exp $
 */
#ifndef __GSF_MEMARR_H__
#define __GSF_MEMARR_H__ 1

#include <glib.h>

/* FIX: Commented out due to annoing Emacs indent:
#ifdef __cplusplus
extern "C" {
#endif
*/

typedef struct _GsfMemArr GsfMemArr;

struct _GsfMemArr
{
    gpointer next_free;
    guint element_size;
    GSList *chunk_list;
    guint next_chunk_size;
};

GsfMemArr* gsf_memarr_create(guint element_size);
void       gsf_memarr_init(GsfMemArr *ma, guint element_size);
void       gsf_memarr_destroy(GsfMemArr *ma);
void       gsf_memarr_addchunk(GsfMemArr *ma);
gpointer   gsf_memarr_alloc(GsfMemArr *ma);
void       gsf_memarr_free(GsfMemArr *ma, gpointer p);

#define GSF_MA_INITVALUES(type)  {NULL, sizeof(type), NULL, 0}
#define GSF_MA_CREATE(type)      gsf_memarr_create(sizeof(type))
#define GSF_MA_INIT(ma, type)    gsf_memarr_init((ma), sizeof(type))
#define GSF_MA_DESTROY(ma)       gsf_memarr_destroy(ma)

#if defined (__GNUC__) && !defined (__STRICT_ANSI__)

#  define GSF_MA_ALLOC(ma)         ({		\
    gpointer __p = (ma)->next_free;		\
    if (!__p)					\
	__p = gsf_memarr_alloc(ma);		\
    else					\
	(ma)->next_free = *((gpointer *)__p);	\
    __p;					\
})

#else

#  define GSF_MA_ALLOC(ma)       gsf_memarr_alloc(ma)

#endif
     
#define GSF_MA_FREE(ma, p)       G_STMT_START {	\
    *((gpointer *)(p)) = (ma)->next_free;	\
    (ma)->next_free = (p);			\
} G_STMT_END


#define GSF_MA_NEW(ma, type)     ((type *)GSF_MA_ALLOC(ma))

/* FIX: Commented out due to annoing Emacs indent:
#ifdef __cplusplus
}
#endif
*/



#endif /* __GSF_MEMARR_H__ */
