/* GSF - GTK+ SpriteField.  Library to use sprites in GTK+
 * Copyright (C) 1998  Tore Skjellnes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
/*
 * Painter
 *
 * $Id: gsfpainter.h,v 1.7 1998/07/30 03:35:41 torsk Exp $
 */
#ifndef __GSF_PAINTER_H__
#define __GSF_PAINTER_H__ 1

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glib.h>

#include "gsfrectangle.h"

/* FIX: Commented out due to annoing Emacs indent:
#ifdef __cplusplus
extern "C" {
#endif
*/

#define GSF_PAINTER(obj)             (GTK_CHECK_CAST (obj, gsf_painter_get_type (), GsfPainter))
#define GSF_PAINTER_CLASS(klass)     (GTK_CHECK_CLASS_CAST (klass, gsf_painter_get_type (), GsfPainterClass))
#define GSF_IS_PAINTER(obj)          (GTK_CHECK_TYPE (obj, gsf_painter_get_type ()))

typedef struct _GsfPainter           GsfPainter;
typedef struct _GsfPainterClass      GsfPainterClass;

struct _GsfPainter
{
    GtkObject object;
    
    GdkDrawable *drawable;
    GdkColormap *colormap;
    GdkGC *gc;
    gint originx;
    gint originy;
};

struct _GsfPainterClass
{
    GtkObjectClass parent_class;
};

GtkType           gsf_painter_get_type        (void);
GsfPainter*       gsf_painter_new             (void);
void              gsf_painter_ref             (GsfPainter *painter);
void              gsf_painter_unref           (GsfPainter *painter);
void              gsf_painter_set_drawable    (GsfPainter *painter,
					       GdkDrawable *drawable,
					       GdkColormap *colormap,
					       GdkGC *gc);
void              gsf_painter_init_pixmap     (GsfPainter *painter,
					       GtkWidget *widget,
					       guint width,
					       guint height);
void              gsf_painter_set_origin      (GsfPainter *p,
					       gint originx,
					       gint originy);
void              gsf_painter_get_origin      (GsfPainter *p,
					       gint *originx,
					       gint *originy);
void              gsf_painter_alloc_gdkcolor  (GsfPainter *p,
					       GdkColor *c);
void              gsf_painter_alloc_rgbcolor  (GsfPainter *p,
					       guint16 r,
					       guint16 g,
					       guint16 b,
					       GdkColor *c);
void              gsf_painter_set_foreground  (GsfPainter *p,
					       GdkColor *c);
void              gsf_painter_set_background  (GsfPainter *p,
					       GdkColor *c);
void              gsf_painter_fill_rectangle  (GsfPainter *p,
					       GsfRectangle *area);
void              gsf_painter_draw_tiled_pixmap(GsfPainter *p,
						GdkPixmap *pixmap,
						gint offsetx,
						gint offsety,
						GsfRectangle *area);
void              gsf_painter_draw_pixmap     (GsfPainter *p,
					       GdkPixmap *pixmap,
					       GsfRectangle *area);
void              gsf_painter_draw_masked_tiled_pixmap(GsfPainter *p,
						       GdkPixmap *pixmap,
						       GdkBitmap *mask,
						       gint offsetx,
						       gint offsety,
						       GsfRectangle *area);
void              gsf_painter_draw_masked_pixmap(GsfPainter *p,
						 GdkPixmap *pixmap,
						 GdkBitmap *mask,
						 GsfRectangle *area);
					       
/* FIX: Commented out due to annoing Emacs indent:
#ifdef __cplusplus
}
#endif
*/  

#endif /* __GSF_PAINTER_H__ */
