/* GSF - GTK+ SpriteField.  Library to use sprites in GTK+
 * Copyright (C) 1998  Tore Skjellnes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
/*
 * PixmapSprite
 *
 * $Id: gsfpixmapsprite.h,v 1.6 1998/07/30 03:35:42 torsk Exp $
 */
#ifndef __GSF_PIXMAPSPRITE_H__
#define __GSF_PIXMAPSPRITE_H__ 1

#include <gtk/gtk.h>
#include <glib.h>
#include "gsfbasesprite.h"
#include "gsfspritegfx.h"

/* FIX: Commented out due to annoing Emacs indent:
#ifdef __cplusplus
extern "C" {
#endif
*/

#define GSF_PIXMAPSPRITE(obj)             (GTK_CHECK_CAST (obj, gsf_pixmapsprite_get_type (), GsfPixmapSprite))
#define GSF_PIXMAPSPRITE_CLASS(klass)     (GTK_CHECK_CLASS_CAST (klass, gsf_pixmapsprite_get_type (), GsfPixmapSpriteClass))
#define GSF_IS_PIXMAPSPRITE(obj)          (GTK_CHECK_TYPE (obj, gsf_pixmapsprite_get_type ()))

typedef struct _GsfPixmapSpriteClass GsfPixmapSpriteClass;
typedef struct _GsfPixmapSprite      GsfPixmapSprite;

struct _GsfPixmapSprite
{
    GsfBaseSprite basesprite;

    gint x;
    gint y;
    gint currentframe;
    
    gint numframes;
    GsfSpriteGfx **gfx;
};

struct _GsfPixmapSpriteClass
{
    GsfBaseSpriteClass parent_class;
};

GtkType           gsf_pixmapsprite_get_type       (void);
GsfPixmapSprite*  gsf_pixmapsprite_new            (void);
void              gsf_pixmapsprite_ref            (GsfPixmapSprite *sprite);
void              gsf_pixmapsprite_unref          (GsfPixmapSprite *sprite);
void              gsf_pixmapsprite_set            (GsfPixmapSprite *sprite,
						   gint x,
						   gint y,
						   gint frame,
						   GsfSpriteGfx **gfx,
						   gint numframes,
						   GsfSpriteField *f,
						   gint z);
void              gsf_pixmapsprite_setxyf         (GsfPixmapSprite *sprite,
						   gint x,
						   gint y,
						   gint frame);
void              gsf_pixmapsprite_getsize        (GsfPixmapSprite *sprite,
						   gint *width,
						   gint *height);
void              gsf_pixmapsprite_pastegfxrect   (GsfPixmapSprite *dest,
						   GsfPixmapSprite *src,
						   gint destx,
						   gint desty,
						   GsfRectangle *srcarea,
						   gboolean maskedcopy);
void              gsf_pixmapsprite_pastegfx       (GsfPixmapSprite *dest,
						   GsfPixmapSprite *src,
						   gint destx,
						   gint desty);
void              gsf_pixmapsprite_clearrect      (GsfPixmapSprite *dest,
						   GsfRectangle *destarea);

#define gsf_pixmapsprite_moveto(s, x, y)			\
                gsf_pixmapsprite_setxyf((s), 			\
					(x), 			\
					(y), 			\
					(s)->currentframe);
#define gsf_pixmapsprite_moveby(s, dx, dy) 			\
                gsf_pixmapsprite_setxyf((s), 			\
					(s)->x + (x), 		\
					(s)->y + (y), 		\
					(s)->currentframe);
#define gsf_pixmapsprite_frame(s, f)		\
                gsf_pixmapsprite_setxyf((s),	\
					(s)->x,	\
					(s)->y,	\
					(f));

/* FIX: Commented out due to annoing Emacs indent:
#ifdef __cplusplus
}
#endif
*/  

#endif /* __GSF_PIXMAPSPRITE_H__ */
