/* GSF - GTK+ SpriteField.  Library to use sprites in GTK+
 * Copyright (C) 1998  Tore Skjellnes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "gsfrectangle.h"

GsfMemArr gsf_rectangle_memarr = GSF_MA_INITVALUES(GsfRectangle);


GsfRectangle *
gsf_rectangle_clone(GsfRectangle *source)
{
    GsfRectangle *dest = GSF_RECTANGLE_NEW();
    *dest = *source;
    return dest;
}

void
gsf_rectangle_set(GsfRectangle *dest,
		  gint x,
		  gint y,
		  guint width,
		  guint height)
{
    dest->x = x;
    dest->y = y;
    dest->width = width;
    dest->height = height;
}

void
gsf_rectangle_set_corners(GsfRectangle *dest,
			  gint x1,
			  gint y1,
			  gint x2,
			  gint y2)
{
    if (x1 < x2) {
	dest->x = x1;
	dest->width = x2 - x1 + 1;
    } else {
	dest->x = x2;
	dest->width = x1 - x2 + 1;
    }
    
    if (y1 < y2) {
	dest->y = y1;
	dest->height = y2 - y1 + 1;
    } else {
	dest->y = y2;
	dest->height = y1 - y2 + 1;
    }
}

gboolean
gsf_rectangle_isempty(GsfRectangle *source)
{
    return source->width == 0 || source->height == 0;
}

gboolean
gsf_rectangle_intersects(GsfRectangle *source1,
			 GsfRectangle *source2)
{
    gint delta;

    delta = source2->x - source1->x;
    if (delta >= (gint)source1->width || -delta >= (gint)source2->width)
	return FALSE;
    
    delta = source2->y - source1->y;
    return (delta < (gint)source1->height && -delta < (gint)source2->height);
}
    
void
gsf_rectangle_makebound(GsfRectangle *dest,
			GsfRectangle *source1,
			GsfRectangle *source2)
{
    gint delta;

    delta = source2->x - source1->x;
    if (delta > 0) {
	delta += source2->width;
	dest->x = source1->x;
	dest->width = MAX(delta, source1->width);
    } else {
	delta = source1->width - delta;
	dest->x = source2->x;
	dest->width = MAX(delta, source2->width);
    }

    delta = source2->y - source1->y;
    if (delta > 0) {
	delta += source2->height;
	dest->y = source1->y;
	dest->height = MAX(delta, source1->height);
    } else {
	delta = source1->height - delta;
	dest->y = source2->y;
	dest->height = MAX(delta, source2->height);
    }
}

void
gsf_rectangle_intersection(GsfRectangle *dest,
			   GsfRectangle *source1,
			   GsfRectangle *source2)
{
    gint delta;

    delta = source2->x - source1->x;
    if (delta > 0) {
	delta = (gint)source1->width - delta;
	dest->width = delta < 0 ? 0 : MIN(delta, source2->width);
	dest->x = source2->x;
    } else {
	delta += (gint)source2->width;
	dest->width = delta < 0 ? 0 : MIN(delta, source1->width);
	dest->x = source1->x;
    }

    delta = source2->y - source1->y;
    if (delta > 0) {
	delta = (gint)source1->height - delta;
	dest->height = delta < 0 ? 0 : MIN(delta, source2->height);
	dest->y = source2->y;
    } else {
	delta += (gint)source2->height;
	dest->height = delta < 0 ? 0 : MIN(delta, source1->height);
	dest->y = source1->y;
    }
}


