/* GSF - GTK+ SpriteField.  Library to use sprites in GTK+
 * Copyright (C) 1998  Tore Skjellnes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
/*
 * SpriteField
 *
 * $Id: gsfspritefield.h,v 1.8 1998/07/30 03:35:42 torsk Exp $
 */
#ifndef __GSF_SPRITEFIELD_H__
#define __GSF_SPRITEFIELD_H__ 1

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glib.h>

#include "gsfrectangle.h"
#include "gsfpainter.h"

/* FIX: Commented out due to annoing Emacs indent:
#ifdef __cplusplus
extern "C" {
#endif
*/

#define GSF_SPRITEFIELD(obj)             (GTK_CHECK_CAST (obj, gsf_spritefield_get_type (), GsfSpriteField))
#define GSF_SPRITEFIELD_CLASS(klass)     (GTK_CHECK_CLASS_CAST (klass, gsf_spritefield_get_type (), GsfSpriteFieldClass))
#define GSF_IS_SPRITEFIELD(obj)          (GTK_CHECK_TYPE (obj, gsf_spritefield_get_type ()))

#include "gsftypedefs.h"

typedef void (* GsfSpriteFieldDrawFunc)(GsfSpriteField *field,
					GsfPainter *p,
					GsfRectangle *area);

typedef struct _GsfSpriteCell          GsfSpriteCell;

struct _GsfSpriteCell
{
    GSList *sprites;
    gboolean clean;
};

struct _GsfSpriteField
{
    GtkObject object;

    guint width;
    guint height;
    guint cellcols;
    guint cellrows;
    guint cellsize;

    GdkColor bg_color;
    GdkPixmap *bg_pixmap;
    GsfSpriteFieldDrawFunc drawbackground;
    GsfSpriteFieldDrawFunc drawforeground;

    GsfSpriteCell *cells;
    GSList *all_objects;
    GSList *view_list;
};


struct _GsfSpriteFieldClass
{
    GtkObjectClass parent_class;
    
    void (* drawgraphics)  (GsfSpriteField *field,
			    GsfRectangle *area,
			    GSList *sprites,
		  	    GSList *views);
    
    /* signals: */
    void (* changed)       (GsfSpriteField *field);
};

GtkType           gsf_spritefield_get_type       (void);
GsfSpriteField*   gsf_spritefield_new            (guint width,
						  guint height,
						  guint cellsize);
void              gsf_spritefield_ref            (GsfSpriteField *f);
void              gsf_spritefield_unref          (GsfSpriteField *f);
void              gsf_spritefield_resize_c       (GsfSpriteField *f,
						  guint width,
						  guint height,
						  guint cellsize);
void              gsf_spritefield_resize         (GsfSpriteField *f,
						  guint width,
						  guint height);
void              gsf_spritefield_setbgcolor     (GsfSpriteField *f,
						  GdkColor *c);
void              gsf_spritefield_setbgpixmap    (GsfSpriteField *f,
						  GdkPixmap *p);
void              gsf_spritefield_setbgfunction  (GsfSpriteField *f,
						  GsfSpriteFieldDrawFunc d);
void              gsf_spritefield_setfgfunction  (GsfSpriteField *f,
						  GsfSpriteFieldDrawFunc d);


void              gsf_spritefield_addsprite      (GsfSpriteField *f,
						  GsfBaseSprite *s);
void              gsf_spritefield_removesprite   (GsfSpriteField *f,
						  GsfBaseSprite *s);
void              gsf_spritefield_removeallsprites(GsfSpriteField *f);

void              gsf_spritefield_addview        (GsfSpriteField *f,
						  GsfSpriteView *s);
void              gsf_spritefield_removeview     (GsfSpriteField *f,
						  GsfSpriteView *s);
void              gsf_spritefield_removeallviews (GsfSpriteField *f);

void              gsf_spritefield_setchanged     (GsfSpriteField *f,
						  GsfRectangle *area,
						  GsfBaseSprite *s,
						  gboolean add);
void              gsf_spritefield_movechanged    (GsfSpriteField *f,
						  GsfRectangle *oldarea,
						  GsfRectangle *newarea,
						  GsfBaseSprite *s);

void              gsf_spritefield_update         (GsfSpriteField *f);
void              gsf_spritefield_updateview     (GsfSpriteField *f,
						  GsfRectangle *area,
						  GsfSpriteView *v);

void              gsf_spritefield_drawareaviews  (GsfSpriteField *f,
						  gboolean force,
						  GsfRectangle *area,
						  GSList *views);
void              gsf_spritefield_drawarea       (GsfSpriteField *f,
						  gboolean force,
						  GsfRectangle *area);
void              gsf_spritefield_drawareaview   (GsfSpriteField *f,
						  gboolean force,
						  GsfRectangle *area,
						  GsfSpriteView *v);

GSList *          gsf_spritefield_approx_at      (GsfSpriteField *f,
						  gint x,
						  gint y);
GSList *          gsf_spritefield_approx_at_rect (GsfSpriteField *f,
						  GsfRectangle *area);
GSList *          gsf_spritefield_at             (GsfSpriteField *f,
					          gint x,
						  gint y);
GSList *          gsf_spritefield_get_collisions (GsfSpriteField *f);

/* FIX: Commented out due to annoing Emacs indent:
#ifdef __cplusplus
}
#endif
*/  

#endif /* __GSF_SPRITEFIELD_H__ */
