/* GSF - GTK+ SpriteField.  Library to use sprites in GTK+
 * Copyright (C) 1998  Tore Skjellnes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
/*
 * Gsf sprite graphics
 *
 * $Id: gsfspritegfx.h,v 1.9 1998/07/30 03:35:42 torsk Exp $
 */
#ifndef __GSF_SPRITEGFX_H__
#define __GSF_SPRITEGFX_H__ 1

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glib.h>

#include "gsfrectangle.h"
#include "gsfcollisionmask.h"

/* FIX: Commented out due to annoing Emacs indent:
#ifdef __cplusplus
extern "C" {
#endif
*/

#define GSF_SPRITEGFX(obj)             (GTK_CHECK_CAST (obj, gsf_spritegfx_get_type (), GsfSpriteGfx))
#define GSF_SPRITEGFX_CLASS(klass)     (GTK_CHECK_CLASS_CAST (klass, gsf_spritegfx_get_type (), GsfSpriteGfxClass))
#define GSF_IS_SPRITEGFX(obj)          (GTK_CHECK_TYPE (obj, gsf_spritegfx_get_type ()))

typedef struct _GsfSpriteGfxClass GsfSpriteGfxClass;
typedef struct _GsfSpriteGfx      GsfSpriteGfx;

struct _GsfSpriteGfx
{
    GtkObject object;

    GdkPixmap *pixmap;
    GdkBitmap *mask;

    guint width, height;
    gint hotx, hoty;

    GsfCollisionMask *collmask;
};

struct _GsfSpriteGfxClass
{
    GtkObjectClass parent_class;

    void (* pixmap_to_coll)(gint *x, gint *y);
};

GtkType           gsf_spritegfx_get_type       (void);
GsfSpriteGfx*     gsf_spritegfx_new            (void);
void              gsf_spritegfx_ref            (GsfSpriteGfx *spritegfx);
void              gsf_spritegfx_unref          (GsfSpriteGfx *spritegfx);

void              gsf_spritegfx_set            (GsfSpriteGfx *spritegfx,
						GdkPixmap *pixmap,
						GdkBitmap *mask);
void              gsf_spritegfx_set_coll       (GsfSpriteGfx *spritegfx,
						GdkPixmap *pixmap,
						GdkBitmap *mask,
						GdkImage *collmask);
gboolean          gsf_spritegfx_collides       (GsfSpriteGfx *spritegfx1,
						gint x1,
						gint y1,
						GsfSpriteGfx *spritegfx2,
						gint x2,
						gint y2);
gboolean          gsf_spritegfx_collides_at    (GsfSpriteGfx *spritegfx,
						gint x,
						gint y);
void              gsf_spritegfx_pastegfxrect   (GsfSpriteGfx *dest,
						GsfSpriteGfx *src,
						gint destx,
						gint desty,
						GsfRectangle *srcarea,
						gboolean maskedcopy);
void              gsf_spritegfx_pastegfx       (GsfSpriteGfx *dest,
						GsfSpriteGfx *src,
						gint destx,
						gint desty);
void              gsf_spritegfx_clearrect      (GsfSpriteGfx *dest,
						GsfRectangle *area);
void              gsf_spritegfx_p2c            (GsfSpriteGfx *gfx,
						gint *x,
						gint *y);
void              gsf_spritegfx_p2c_rect       (GsfSpriteGfx *gfx,
						gint *x,
						gint *y,
						guint *width,
						guint *height);
void              gsf_spritegfx_set_collconvert(guint mul, guint div);
void              gsf_spritegfx_get_collconvert(guint *mul, guint *div);
void              gsf_spritegfx_scale_collmask (GsfSpriteGfx *spritegfx);

/* FIX: Commented out due to annoing Emacs indent:
#ifdef __cplusplus
}
#endif
*/  

#endif /* __GSF_SPRITEGFX_H__ */
