/* GSF - GTK+ SpriteField.  Library to use sprites in GTK+
 * Copyright (C) 1998  Tore Skjellnes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "gsfspriteview.h"
#include "gsfspritefield.h"

static void gsf_spriteview_class_init (GsfSpriteViewClass *klass);
static void gsf_spriteview_init       (GsfSpriteView      *spriteview);
static void gsf_spriteview_destroy    (GtkObject          *object);
static gint gsf_spriteview_expose     (GtkWidget          *widget,
				       GdkEventExpose     *event);

static void gsf_spriteview_getviewarea(GsfSpriteView      *s,
				       GsfRectangle       *area);
				       
static GtkDrawingAreaClass *parent_class = NULL;

GtkType
gsf_spriteview_get_type(void)
{
    static GtkType spriteview_type = 0;

    if (!spriteview_type) {
	GtkTypeInfo spriteview_info = {
	    "GsfSpriteView",
	    sizeof(GsfSpriteView),
	    sizeof(GsfSpriteViewClass),
	    (GtkClassInitFunc) gsf_spriteview_class_init,
	    (GtkObjectInitFunc) gsf_spriteview_init,
	    (GtkArgSetFunc) NULL,
	    (GtkArgGetFunc) NULL,
	};

	spriteview_type = gtk_type_unique(gtk_drawing_area_get_type(),
					  &spriteview_info);
    }

    return spriteview_type;
}


static void
gsf_spriteview_class_init(GsfSpriteViewClass *klass)
{
    GtkObjectClass *object_class;
    GtkWidgetClass *widget_class;

    object_class = (GtkObjectClass*)klass;
    widget_class = (GtkWidgetClass*)klass;
    parent_class = gtk_type_class(gtk_drawing_area_get_type());
    
    object_class->destroy = gsf_spriteview_destroy;

    widget_class->expose_event = gsf_spriteview_expose;    
    klass->getviewarea = gsf_spriteview_getviewarea;
}

static void
gsf_spriteview_init(GsfSpriteView *spriteview)
{
    spriteview->field = NULL;
}

GtkWidget *
gsf_spriteview_new(void)
{
    GsfSpriteView *spriteview;

    spriteview = gtk_type_new(gsf_spriteview_get_type());

    return GTK_WIDGET(spriteview);
}

static void
gsf_spriteview_destroy(GtkObject *object)
{
    GsfSpriteView *spriteview;

    g_return_if_fail(object != NULL && GSF_IS_SPRITEVIEW(object));

    spriteview = GSF_SPRITEVIEW(object);

    gsf_spriteview_setfield(spriteview, NULL);

    if (GTK_OBJECT_CLASS(parent_class)->destroy)
	(* GTK_OBJECT_CLASS(parent_class)->destroy)(object);
}

void
gsf_spriteview_setfield(GsfSpriteView *s, GsfSpriteField *f)
{
    g_return_if_fail(s != NULL && GSF_IS_SPRITEVIEW(s));
    g_return_if_fail(f == NULL || GSF_IS_SPRITEFIELD(f));

    if (f)
	gtk_widget_ref(GTK_WIDGET(s));
    if (s->field)
	gsf_spritefield_removeview(s->field, s);
    if (f) {
	gsf_spritefield_addview(f, s);
	gtk_drawing_area_size(GTK_DRAWING_AREA(s),
			      f->width,
			      f->height);
	gtk_widget_unref(GTK_WIDGET(s));
    }
}



static void
gsf_spriteview_getviewarea(GsfSpriteView *s,
			   GsfRectangle  *area)
{
    GtkWidget *w;
    g_return_if_fail(s != NULL && GSF_IS_SPRITEVIEW(s));

    w = GTK_WIDGET(s);
    
    area->x = 0;
    area->y = 0;
    area->width = w->allocation.width;
    area->height = w->allocation.height;
}


static gint
gsf_spriteview_expose(GtkWidget *widget,
		      GdkEventExpose *event)
{
    GsfSpriteView *view;

    g_return_val_if_fail(widget != NULL && GSF_IS_SPRITEVIEW(widget), FALSE);
    g_return_val_if_fail(event != NULL, FALSE);

    if (GTK_WIDGET_DRAWABLE(widget)) {
	view = GSF_SPRITEVIEW(widget);

	if (view->field) {
	    GsfRectangle area;
	    
	    area.x = event->area.x;
	    area.y = event->area.y;
	    area.width = event->area.width;
	    area.height = event->area.height;
	    
	    gsf_spritefield_updateview(view->field, &area, view);
	}
    }
    return FALSE;
}
